<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) {
    exit;
}
//----------------------------------------------------------------------------------------
// Load translations
//----------------------------------------------------------------------------------------
add_action( 'plugins_loaded', 'teo_load_plugin_textdomain' );
function teo_load_plugin_textdomain() {
    load_plugin_textdomain( 'events-optimizer', false, plugin_basename( dirname( __FILE__ ) ) . '/languages/' );
}

//----------------------------------------------------------------------------------------
// Initialize menu and settings
//----------------------------------------------------------------------------------------
// Add Optimizer to Events Calendar menu in WordPress dashboard
add_action( 'admin_menu', 'teo_initialize_menu' );
function teo_initialize_menu() {
    // Check if the user has the necessary permissions
    if ( !(current_user_can( 'administrator' ) or current_user_can( sanitize_text_field( get_option( 'teo_allowed_role' ) ) )) ) {
        wp_die( esc_html__( 'You do not have sufficient permissions to access this page.', 'events-optimizer' ) );
    }
    // Add Menu to Events Calendar
    $my_page = add_submenu_page(
        'edit.php?post_type=tribe_events',
        'Events Optimizer',
        'Optimizer',
        'administrator',
        'events_optimizer',
        'events_optimizer_menu'
    );
    // Call register settings function
    add_action( 'admin_init', 'teo_register_settings' );
    // Load the JS conditionally (see next function below)
    add_action( 'load-' . $my_page, 'teo_load_admin_js' );
}

//----------------------------------------------------------------------------------------
// This function is only called when the Event Optimizers page is loaded
// https://wordpress.stackexchange.com/questions/41207/how-do-i-enqueue-styles-scripts-on-certain-wp-admin-pages
//----------------------------------------------------------------------------------------
function teo_load_admin_js() {
    // Register against the proper action hook
    add_action( 'admin_enqueue_scripts', 'teo_enqueue_admin_js' );
}

//----------------------------------------------------------------------------------------
// Include CSS and JavaScript files only when the Event Optimizer page is loaded
//----------------------------------------------------------------------------------------
function teo_enqueue_admin_js() {
    // Load the scripts that have been registered above
    wp_enqueue_script( 'teo-tabs', plugin_dir_url( __FILE__ ) . 'assets/js/tabs.js' );
    wp_enqueue_style( 'teo-style', plugin_dir_url( __FILE__ ) . 'assets/css/style.css' );
}

// Class for creating a standard WordPress table with pagination
if ( !class_exists( 'WP_List_Table' ) ) {
    require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}
//----------------------------------------------------------------------------------------
// Register settings
//----------------------------------------------------------------------------------------
function teo_register_settings() {
    register_setting( 'events-optimizer-settings', 'teo_max_results_per_page' );
    //register_setting( 'events-optimizer-settings', 'teo_remove_settings' );
    // Set default values
    if ( !get_option( 'teo_max_results_per_page' ) ) {
        update_option( 'teo_max_results_per_page', 5 );
    }
    // endif teo_fs()->can_use_premium_code__premium_only()
}

//----------------------------------------------------------------------------------------
// Register and enque scripts for admin notices
//----------------------------------------------------------------------------------------
add_action( 'admin_enqueue_scripts', 'add_scripts' );
function add_scripts() {
    // Scripts are automatically registered when they are enqueued
    wp_enqueue_script(
        'sanitize',
        plugin_dir_url( __FILE__ ) . 'assets/js/sanitize.js',
        array(),
        '1.0',
        false
    );
    wp_enqueue_script(
        'admin-notices',
        plugin_dir_url( __FILE__ ) . 'assets/js/admin-notices.js',
        array(),
        '1.0',
        false
    );
    wp_localize_script( 'admin-notices', 'notice_params', array(
        'ajaxurl' => admin_url( 'admin-ajax.php' ),
    ) );
}

//----------------------------------------------------------------------------------------
// After new installation activated or update show notices in WordPress Dashboard
//----------------------------------------------------------------------------------------
register_activation_hook( __FILE__, 'show_notice_plugin_activated' );
function show_notice_plugin_activated() {
    // Set flag that the plugin has been installed and activated
    update_option( 'teo_notice_plugin_activated', true );
}

add_action(
    'upgrader_process_complete',
    'show_notice_plugin_updated',
    10,
    2
);
function show_notice_plugin_updated(  $upgrader_object, $options  ) {
    $current_plugin_path_name = plugin_basename( __FILE__ );
    if ( $options['action'] == 'update' && $options['type'] == 'plugin' && isset( $options['plugins'] ) ) {
        foreach ( $options['plugins'] as $plugin ) {
            if ( $plugin == $current_plugin_path_name ) {
                // Set flag that the plugin has been updated
                update_option( 'teo_notice_plugin_updated', true );
                break;
            }
        }
    }
}

//----------------------------------------------------------------------------------------
// Messages to show in WordPress Dashboard
//----------------------------------------------------------------------------------------
add_action( 'admin_notices', 'admin_notices' );
function admin_notices() {
    // Check if The Events Calendar plugin is installed and active
    if ( is_plugin_active( 'the-events-calendar/the-events-calendar.php' ) ) {
        // After first activation show link to plugin settings
        if ( get_option( 'teo_notice_plugin_activated' ) ) {
            ?><div class="notice notice-success teo-notice-plugin-activated is-dismissible">
					<p>
					<?php 
            esc_html_e( 'Events Optimizer installation was successful! To start set up the plugin now in the', 'events-optimizer' );
            ?>
					<a href="<?php 
            echo esc_url( admin_url( 'edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-4' ) );
            ?>"> 
					<?php 
            esc_html_e( 'Events Optimizer settings', 'events-optimizer' );
            ?></a>
					</p>			
				</div><?php 
        }
        // After updating to a new version show release notes
        if ( get_option( 'teo_notice_plugin_updated' ) ) {
            // Get release notes from readme.txt
            $readme = file_get_contents( plugin_dir_path( __FILE__ ) . 'readme.txt' );
            $start = strpos( $readme, '= 1.5.1 =' ) + 10;
            $end = strpos( $readme, '= 1.5.0 =' );
            $version_info = substr( $readme, $start, $end - $start );
            // Output the HTML
            echo '<div class="notice notice-success teo-notice-plugin-updated is-dismissible">
					<p>' . esc_html__( 'What\'s new in this version of Events Optimizer?', 'events-optimizer' ) . '</p>
					<p>' . nl2br( esc_html( $version_info ) ) . '</p>
					<p>' . esc_html__( 'Find more information in the', 'events-optimizer' ) . ' <a href="https://www.heinrich.biz/change-log/" target="_blank" rel="noopener noreferrer">' . esc_html__( 'change log', 'events-optimizer' ) . '</a>.</p>
				</div>';
        }
        // Check if WordPress language setting is equal to the language setting for the Google Places API
        if ( esc_html( get_option( 'teo_google_places_language' ) ) != substr( get_bloginfo( 'language' ), 0, 2 ) ) {
            ?><div class="notice notice-warning teo-google-places-language-notice is-dismissible">
					<p>
					<?php 
            esc_html_e( 'The language setting for the Google Places API is not equal to the language setting of your WordPress installation.', 'events-optimizer' );
            ?><br>
					<?php 
            esc_html_e( 'This will causes problems when retrieving data from Google Places! Please check the', 'events-optimizer' );
            ?>
					<a href="<?php 
            echo esc_url( admin_url( 'edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-4' ) );
            ?>">
					<?php 
            esc_html_e( 'Events Optimizer settings.', 'events-optimizer' );
            ?>
					</a>
					</p>            
				</div><?php 
        }
    } else {
        // If The Events Calendar plugin is not activated show an error message
        ?><div class="notice notice-error teo-tec-not-installed-notice">
				<p>
				<?php 
        esc_html_e( 'The Events Calendar Plugin is not activated! Please ', 'events-optimizer' );
        ?>
				<a href="<?php 
        echo esc_url( admin_url( 'plugin-install.php?tab=plugin-information&plugin=the-events-calendar&TB_iframe=true&width=772&height=778' ) );
        ?>" class="thickbox">			
				<?php 
        esc_html_e( 'install and activate ', 'events-optimizer' );
        ?></a> 
				<?php 
        esc_html_e( 'it before using the Events Optimizer!', 'events-optimizer' );
        ?>
			</p>			
			</div><?php 
    }
}

//----------------------------------------------------------------------------------------
// If notices are dismissed don't show it again:
// use AJAX to dismiss the notice ( => admin-notices.js)
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_teo_dismiss_notice_plugin_activated', 'teo_dismiss_notice_plugin_activated' );
function teo_dismiss_notice_plugin_activated() {
    update_option( 'teo_notice_plugin_activated', false );
}

add_action( 'wp_ajax_teo_dismiss_notice_plugin_updated', 'teo_dismiss_notice_plugin_updated' );
function teo_dismiss_notice_plugin_updated() {
    update_option( 'teo_notice_plugin_updated', false );
}

//----------------------------------------------------------------------------------------
// Main function: Create Menu / Tabs
//----------------------------------------------------------------------------------------
function events_optimizer_menu() {
    // Check if the user has the necessary permissions
    if ( !(current_user_can( 'administrator' ) or current_user_can( sanitize_text_field( get_option( 'teo_allowed_role' ) ) )) ) {
        wp_die( esc_html__( 'You do not have sufficient permissions to access this page.', 'events-optimizer' ) );
    }
    // Get settings
    $max_results_per_page = intval( get_option( 'teo_max_results_per_page', 5 ) );
    $phone_number_format = sanitize_text_field( get_option( 'teo_phone_number_format' ) );
    $address_format = sanitize_text_field( get_option( 'teo_address_format' ) );
    $preselection = sanitize_text_field( get_option( 'teo_preselection' ) );
    $nonce = wp_create_nonce( 'events-optimizer-nonce' );
    // Create submenu
    ?>
		<div class='wrap'>	
			<h1 class="wp-heading-inline"><?php 
    esc_html_e( 'Events Optimizer', 'events-optimizer' );
    ?></h1>
		
			<?php 
    if ( version_compare( phpversion(), '8.0', '>=' ) ) {
        // PHP 8.0 and above
        $active_tab = filter_input( INPUT_GET, 'tab', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
    } else {
        // PHP 7.4 and below
        $active_tab = filter_input( INPUT_GET, 'tab', FILTER_SANITIZE_STRING );
    }
    if ( !in_array( $active_tab, [
        'tab-1',
        'tab-2',
        'tab-3',
        'tab-4'
    ] ) ) {
        $active_tab = 'tab-1';
    }
    ?>

			<!-- Setup Tabs -->
			<!-- // https://epiph.yt/blog/2018/einfache-javascript-tabs-im-wordpress-backend/ -->
			<h2 class="nav-tab-wrapper">
				<a href="edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-1" class="nav-tab<?php 
    echo ( $active_tab === 'tab-1' || empty( $active_tab ) ? ' nav-tab-active' : '' );
    ?>" data-slug= 'tab-1' ><?php 
    esc_html_e( 'Venues', 'events-optimizer' );
    ?></a>
				<a href="edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-2" class="nav-tab<?php 
    echo ( $active_tab === 'tab-2' ? ' nav-tab-active' : '' );
    ?>" data-slug= 'tab-2' ><?php 
    esc_html_e( 'Organizers', 'events-optimizer' );
    ?></a>
				<a href="edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-3" class="nav-tab<?php 
    echo ( $active_tab === 'tab-3' ? ' nav-tab-active' : '' );
    ?>" data-slug= 'tab-3' ><?php 
    esc_html_e( 'Events', 'events-optimizer' );
    ?></a>
				<a href="edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-4" class="nav-tab<?php 
    echo ( $active_tab === 'tab-4' ? ' nav-tab-active' : '' );
    ?>" data-slug= 'tab-4' ><?php 
    esc_html_e( 'Settings', 'events-optimizer' );
    ?></a>
			</h2>
			

			<!-- Add Venues content to tabs -->
			<div class="nav-tab-content<?php 
    echo ( $active_tab === 'tab-1' || empty( $active_tab ) ? ' nav-tab-content-active' : '' );
    ?>" id="nav-tab-content-tab-1">
				<p><?php 
    esc_html_e( 'Select the data you want to analyze.', 'events-optimizer' );
    ?></p>

				<div class='wrap'>
					<form action='' method='POST' id='venues_form'>                
						<input type='button' name='action' class='button' id='venues_with_invalid_data' value='<?php 
    esc_attr_e( 'Venues with invalid data', 'events-optimizer' );
    ?>' />&nbsp;&nbsp;&nbsp;
						<input type='button' name='action' class='button' id='duplicate_venues' value='<?php 
    esc_attr_e( 'Duplicate venues', 'events-optimizer' );
    ?>' <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?>  />&nbsp;&nbsp;&nbsp; 
						<input type='button' name='action' class='button' id='venues_google_places_optimization' value='<?php 
    esc_attr_e( 'Optimization with Google Places', 'events-optimizer' );
    ?>' <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?>/>&nbsp;&nbsp;&nbsp;  
						<input type='button' name='action' class='button' id='venues_with_inactive_business_operation' value='<?php 
    esc_attr_e( 'Venues with inactive business operation', 'events-optimizer' );
    ?>' <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?>/>&nbsp;&nbsp;&nbsp;  
					</form>
					<p>
					<div id='show_results_venues'></div>
					<div id='loading_tab_venues'><?php 
    esc_html_e( 'ANALYZING', 'events-optimizer' );
    ?></div>
					</p>
				</div>          
			</div>
			

			<!-- Add Organizers content to tabs -->
			<div class="nav-tab-content<?php 
    echo ( $active_tab === 'tab-2' ? ' nav-tab-content-active' : '' );
    ?>" id="nav-tab-content-tab-2">
				<p><?php 
    esc_html_e( 'Select the data you want to analyze.', 'events-optimizer' );
    ?></p>

				<div class='wrap'>
					<form action='' method='POST' id='organizers_form'>
						<input type='button' name='action' class='button' id='organizers_with_invalid_data' value='<?php 
    esc_attr_e( 'Organizers with invalid data', 'events-optimizer' );
    ?>' />&nbsp;&nbsp;&nbsp;
						<input type='button' name='action' class='button' id='duplicate_organizers' value='<?php 
    esc_attr_e( 'Duplicate organizers', 'events-optimizer' );
    ?>' <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?>  />&nbsp;&nbsp;&nbsp; 
						<input type='button' name='action' class='button' id='organizers_google_places_optimization' value='<?php 
    esc_attr_e( 'Optimization with Google Places', 'events-optimizer' );
    ?>' <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />&nbsp;&nbsp;&nbsp;  
					</form>
					<p>
					<div id='show_results_organizers'></div>
					<div id='loading_tab_organizers'><?php 
    esc_html_e( 'ANALYZING', 'events-optimizer' );
    ?></div>
					</p>
				</div>          
			</div>
			
			
			<!-- Add Events content to tabs -->
			<div class="nav-tab-content<?php 
    echo ( $active_tab === 'tab-3' ? ' nav-tab-content-active' : '' );
    ?>" id="nav-tab-content-tab-3">
				<p><?php 
    esc_html_e( 'Select the data you want to analyze or add. Be aware that adding events is a 2-step approach.', 'events-optimizer' );
    ?></p>

				<div class='wrap'>
					<form action='' method='POST' id='events_form'>
						<span class='frame_around'>
							<input type='button' name='action' class='button' id='events_with_invalid_data' value='<?php 
    esc_attr_e( 'Events with invalid data', 'events-optimizer' );
    ?>'/>
						</span>
						<span class='frame_around'>
							<?php 
    esc_html_e( 'Keyword', 'events-optimizer' );
    ?> <input type="text" name="dataforseo_keyword" id="dataforseo_keyword" size="14" value="<?php 
    esc_attr_e( get_option( 'teo_dataforseo_keyword' ) );
    ?>" <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />&nbsp;&nbsp;&nbsp;
							<?php 
    esc_html_e( 'Max. results', 'events-optimizer' );
    ?> <input type="text" name="dataforseo_depth" id="dataforseo_depth" size="2" value="<?php 
    esc_attr_e( get_option( 'teo_dataforseo_depth' ) );
    ?>"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />&nbsp;&nbsp;
							<input type='button' name='action' class='button' id='search_events_with_google_data' value='<?php 
    esc_attr_e( 'Search for events', 'events-optimizer' );
    ?>' <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?>/>&nbsp;&nbsp;&nbsp;&#10132;&nbsp;&nbsp;&nbsp;
							<input type='button' name='action' class='button' id='add_events_with_google_data' value='<?php 
    esc_attr_e( 'Show results', 'events-optimizer' );
    ?>' <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?>/>
						</span>
					</form>
					<p>
					<div id='show_results_events'></div>
					<div id='loading_tab_events'><?php 
    esc_html_e( 'ANALYZING', 'events-optimizer' );
    ?></div>
					</p>
				</div>          
			</div>

				
			<!-- Add Settings content to tabs -->
			<div class="nav-tab-content<?php 
    echo ( $active_tab === 'tab-4' ? ' nav-tab-content-active' : '' );
    ?>" id="nav-tab-content-tab-4">
				<div>
				
					<form action='' method='POST' id='settings_form'>

						<div class="tribe-settings-form-wrap">
							<br><h3><?php 
    esc_html_e( 'General', 'events-optimizer' );
    ?></h3>
							<?php 
    ?>
							<div id="modern-tribe-info"><?php 
    esc_html_e( 'Upgrade to Events Optimizer Pro! Enjoy amazing features like AI boosted generating descriptions for events, venues and organizers, finding duplicate venues or organizers and auto completion of venue and organizer data via Google Places. And change all settings that are greyed out! Get more information at ', 'events-optimizer' );
    ?>
								<a href="https://www.heinrich.biz" target="_blank" rel="noopener noreferrer">
								<?php 
    esc_html_e( 'our website!', 'events-optimizer' );
    ?>
								</a>
							</div>
							<?php 
    ?>						
						
							<table class="form-table">
								<?php 
    ?>
								<tr valign="top">
								<th scope="row"><?php 
    esc_html_e( 'Events Optimizer Pro Licence Key', 'events-optimizer' );
    ?></th>
								<td><a href="https://checkout.freemius.com/mode/dialog/plugin/8391/plan/13936/" class="button" target="_blank" rel="noopener noreferrer"><?php 
    esc_attr_e( 'Get licence key', 'events-optimizer' );
    ?></a></td>
								</tr>
								<?php 
    ?>						
			
								<tr valign="top">
								<th scope="row"><?php 
    esc_html_e( 'Maximum results per page', 'events-optimizer' );
    ?></th>
								<td><input type="text" name="max_results_per_page" id="max_results_per_page" size="5" value="<?php 
    esc_attr_e( get_option( 'teo_max_results_per_page' ) );
    ?>"/>
									<?php 
    esc_html_e( '(max = 20 to avoid time-outs)', 'events-optimizer' );
    ?></td>
								</tr>
								<!--
								<tr valign="top">
								<th scope="row"><?php 
    esc_html_e( 'Remove settings during uninstall?', 'events-optimizer' );
    ?> </th>
								<td><input type="checkbox" name="remove_settings" id="remove_settings" value="1" <?php 
    checked( get_option( 'teo_remove_settings' ), 1 );
    ?>/></td>
								</tr>
								-->
							</table>
				
							<br><h3><?php 
    esc_html_e( 'Duplicate venues / organizers', 'events-optimizer' );
    ?></h3>				
							<div id="modern-tribe-info">
								<?php 
    esc_html_e( 'With the compare value you can control how identical objects are found. The smaller the value the more identical both objects need to be.', 'events-optimizer' );
    ?><br><br>
								<li><?php 
    esc_html_e( 'For venues the name, the address and the first three characters of the ZIP will be used to find identical venues.', 'events-optimizer' );
    ?></li>
								<li><?php 
    esc_html_e( 'For organizers only the name will be used to find identical organizers.', 'events-optimizer' );
    ?></li>				
							</div>
							<table class="form-table">	
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Duplicate organizers compare value', 'events-optimizer' );
    ?></th>
									<td>
										<input type="text" name="duplicate_organizers_levenshtein" id="duplicate_organizers_levenshtein" size="5" value="<?php 
    esc_attr_e( get_option( 'teo_duplicate_organizers_levenshtein' ) );
    ?>"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
										<?php 
    esc_html_e( '(0 = exact match, recommended = 0.15 - 0.3)', 'events-optimizer' );
    ?>
									</td>
								</tr>
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Duplicate venues compare value', 'events-optimizer' );
    ?></th>
									<td>
										<input type="text" name="duplicate_venues_levenshtein" id="duplicate_venues_levenshtein" size="5" value="<?php 
    esc_attr_e( get_option( 'teo_duplicate_venues_levenshtein' ) );
    ?>"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
										<?php 
    esc_html_e( '(0 = exact match, recommended = 0.3 - 0.5)', 'events-optimizer' );
    ?>
									</td>
								</tr>
							</table>

							<br><h3><?php 
    esc_html_e( 'API credentials', 'events-optimizer' );
    ?></h3>
							<div id="modern-tribe-info">
								<?php 
    echo '<li>' . esc_html__( 'To set up the OpenAI API key check out this', 'events-optimizer' ) . ' <a href="https://www.heinrich.biz/events-optimizer/settings/open-ai-platform-setup/" target="_blank" rel="noopener noreferrer">' . esc_html__( 'manual', 'events-optimizer' ) . '</a>. ';
    echo '<a href="https://platform.openai.com/api-keys" target="_blank" rel="noopener noreferrer">' . esc_html__( 'The API keys', 'events-optimizer' ) . '</a> ' . esc_html__( 'and the', 'events-optimizer' ) . ' ';
    echo '<a href="https://openai.com/pricing" target="_blank" rel="noopener noreferrer">' . esc_html__( 'pricing', 'events-optimizer' ) . '</a> ' . esc_html__( 'can be found on the OpenAI website', 'events-optimizer' ) . '</a>.</li>';
    echo '<li>' . esc_html__( 'To set up the Google API keys check out this', 'events-optimizer' ) . ' <a href="https://www.heinrich.biz/events-optimizer/settings/google-cloud-console-setup/" target="_blank" rel="noopener noreferrer">' . esc_html__( 'manual', 'events-optimizer' ) . '</a>. ';
    echo esc_html__( 'We have preset it (if available) with your API key from the', 'events-optimizer' ) . ' ';
    // Path to Events Calendar settings changed from V5.9 to V6.0
    $plugin_arr = get_plugin_data( WP_PLUGIN_DIR . '/the-events-calendar/the-events-calendar.php', false, false );
    if ( $plugin_arr['Version'] < 6 ) {
        echo '<a href="/wp-admin/edit.php?page=tribe-common&tab=addons&post_type=tribe_events" target="_blank" rel="noopener noreferrer">';
    } else {
        echo '<a href="/wp-admin/edit.php?page=tec-events-settings&tab=addons&post_type=tribe_events" target="_blank" rel="noopener noreferrer">';
    }
    echo esc_html__( 'Events Calendar settings', 'events-optimizer' ) . '</a>. ';
    echo esc_html__( 'Find more', 'events-optimizer' ) . ' <a href="https://ai.google.dev/" target="_blank" rel="noopener noreferrer">' . esc_html__( 'documentation on the Google AI website', 'events-optimizer' ) . '</a>.</li>';
    echo '<li>' . esc_html__( 'To set up the DataForSEO API key check out the', 'events-optimizer' ) . ' <a href="https://dataforseo.com/apis/?aff=142846" target="_blank" rel="noopener">' . esc_html__( 'website', 'events-optimizer' ) . '</a> ' . esc_html__( '(free trial with affiliate link).', 'events-optimizer' ) . ' ';
    echo esc_html__( 'Find the API login and password in the', 'events-optimizer' ) . ' <a href="https://app.dataforseo.com/api-dashboard/?aff=142846" target="_blank" rel="noopener"> ' . esc_html__( 'DataForSEO Dashboard', 'events-optimizer' ) . '</a> ';
    echo esc_html__( 'and the', 'events-optimizer' ) . ' <a href="https://dataforseo.com/pricing/serp/serp-api/?aff=142846" target="_blank" rel="noopener"> ' . esc_html__( 'pricing on the DataForSEO website', 'events-optimizer' ) . '</a>. ';
    ?>
							</div>			
							<table class="form-table">			
							<tr valign="top">					
									<th scope="row"><?php 
    esc_html_e( 'OpenAI API key', 'events-optimizer' );
    ?></th>
									<td><input type="text" name="openai_api_key" id="openai_api_key" size="60"  value="<?php 
    echo esc_attr( get_option( 'teo_openai_api_key' ) );
    ?>"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>
							</tr>
							<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Google API key', 'events-optimizer' );
    ?></th>
									<td>
										<input type="text" name="google_places_api_key" id="google_places_api_key" size="44" value="<?php 
    echo esc_attr( get_option( 'teo_google_places_api_key' ) );
    ?>" <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>                        
								</tr>		
							</tr>			
							<tr valign="top">							
								<th scope="row"><?php 
    esc_html_e( 'DataForSEO API login', 'events-optimizer' );
    ?></th>
								<td>
									<input type="text" name="dataforseo_api_login" id="dataforseo_api_login" size="30"  value="<?php 
    echo esc_attr( get_option( 'teo_dataforseo_api_login' ) );
    ?>"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
								</td>
							</tr>	
							<tr valign="top">							
								<th scope="row"><?php 
    esc_html_e( 'DataForSEO API password', 'events-optimizer' );
    ?></th>
								<td>
									<input type="text" name="dataforseo_api_password" id="dataforseo_api_password" size="30"  value="<?php 
    echo esc_attr( get_option( 'teo_dataforseo_api_password' ) );
    ?>"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
								</td>
							</tr>	
							</table>
							<br>

							<br><h3><?php 
    esc_html_e( 'Generate descriptions for events, venues and organizers', 'events-optimizer' );
    ?><span class='beta_version'>BETA</span></h3>
							<div id="modern-tribe-info">
								<?php 
    echo esc_html__( 'To automatically generate descriptions the OpenAI or Google API is used.', 'events-optimizer' ) . "<br><br>";
    echo '<li> ' . esc_html__( 'Activate the free Google Generative Language API for the relevant project in the', 'events-optimizer' ) . ' <a href="https://console.cloud.google.com/marketplace/product/google/generativelanguage.googleapis.com" target="_blank"  rel="noopener noreferrer">' . esc_html__( 'Google Cloud Console', 'events-optimizer' ) . '</a>.</li>';
    echo '<li> ' . esc_html__( 'Google Gemini is not available in EU and UK yet. Check out here the', 'events-optimizer' ) . ' <a href="https://ai.google.dev/gemini-api/docs/available-regions" target="_blank"  rel="noopener noreferrer">' . esc_html__( 'available regions', 'events-optimizer' ) . '</a>.</li>';
    echo '<li><strong>' . esc_html__( 'AI can hallucinate and produce false output! Please do a fact check before publishing it!', 'events-optimizer' ) . '</strong></li>';
    echo '<li>' . esc_html__( 'Sometimes the APIs are not available. You can check here the status of the', 'events-optimizer' ) . ' <a href="https://status.openai.com/" target="_blank" rel="noopener noreferrer">' . esc_html__( 'OpenAI API', 'events-optimizer' ) . '</a> ';
    echo esc_html__( 'and the', 'events-optimizer' ) . ' <a href="https://status.gemini.com/" target="_blank" rel="noopener noreferrer">' . esc_html__( 'Google Gemini API', 'events-optimizer' ) . '</a>.</li>';
    ?>
							</div>
																
							<table class="form-table">

								<tr valign="top">					
								<th scope="row"><?php 
    esc_html_e( 'Model', 'events-optimizer' );
    ?></th>
								<td>
								<select name="openai_model" id="openai_model" <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> >  
								<?php 
    $selected_openai_model = sanitize_text_field( get_option( 'teo_openai_model' ) );
    $openai_model_options = array(
        'gpt-3.5-turbo'       => esc_html__( 'GPT-3.5 turbo', 'events-optimizer' ),
        'gpt-3.5-turbo-0301'  => esc_html__( 'GPT-3.5 turbo (0301)', 'events-optimizer' ),
        'gpt-3.5-turbo-0613'  => esc_html__( 'GPT-3.5 turbo (0613)', 'events-optimizer' ),
        'gpt-3.5-turbo-1106'  => esc_html__( 'GPT-3.5 turbo (1106)', 'events-optimizer' ),
        'gpt-3.5-turbo-16k'   => esc_html__( 'GPT-3.5 turbo 16k', 'events-optimizer' ),
        'gpt-4'               => esc_html__( 'GPT-4', 'events-optimizer' ),
        'gpt-4-1106-preview'  => esc_html__( 'GPT-4 (1106 preview)', 'events-optimizer' ),
        'gpt-4-turbo'         => esc_html__( 'GPT-4 turbo', 'events-optimizer' ),
        'gpt-4-turbo-preview' => esc_html__( 'GPT-4 turbo (preview)', 'events-optimizer' ),
        'gemini-1.0-pro'      => esc_html__( 'Gemini 1.0 pro', 'events-optimizer' ),
    );
    // gpt-3.5-turbo-16k currently points to gpt-3.5-turbo-16k-0613
    //'gpt-3.5-turbo-16k-0613'	=> esc_html__( 'GPT-3.5 turbo 16k (0613)', 'events-optimizer' ),
    // gpt-3.5-turbo currently points to gpt-3.5-turbo-0125
    //'gpt-3.5-turbo-0125'		=> esc_html__( 'GPT-3.5 turbo (0125)', 'events-optimizer' ),
    // gtp-4-turbo currently points to gpt-4-turbo-2024-04-09.
    //'gpt-4-turbo-2024-04-09'	=> esc_html__( 'GPT-4 turbo (2024-04-09)', 'events-optimizer' ),
    // gpt4-turbo currently points to gpt-4-0125-preview.
    //'gpt-4-0125-preview'		=> esc_html__( 'GPT-4 (0125 preview)', 'events-optimizer' ),
    // gpt-4 currently points to gpt-4-0613.
    //'gpt-4-0613'				=> esc_html__( 'GPT-4 (0613)', 'events-optimizer' ),
    // These models were never rolled out widely in favor of GPT-4 Turbo.
    // 'gpt-4-32k-0613'			=> esc_html__( 'GPT-4 32K-0613', 'events-optimizer' ),
    // 'gpt-4-32k'				=> esc_html__( 'GPT-4 32K', 'events-optimizer' ),
    foreach ( $openai_model_options as $openai_model_value => $openai_model_decription ) {
        echo '<option value="' . esc_attr( $openai_model_value ) . '" ' . selected( $selected_openai_model, $openai_model_value, false ) . '>' . esc_attr( $openai_model_decription ) . '</option>';
    }
    ?>
								</select>
								<?php 
    esc_html_e( '(find more details about the available models in the', 'events-optimizer' );
    ?>
								<a href="https://platform.openai.com/docs/models/overview" target="_blank"  rel="noopener noreferrer">
								<?php 
    esc_html_e( 'OpenAI', 'events-optimizer' );
    ?></a> 
								<?php 
    esc_html_e( 'or', 'events-optimizer' );
    ?> 
								<a href="https://cloud.google.com/vertex-ai/docs/generative-ai/learn/model-versioning?hl=de#latest-version" target="_blank"  rel="noopener noreferrer">
								<?php 
    esc_html_e( 'Google', 'events-optimizer' );
    ?></a> 
								<?php 
    esc_html_e( 'documentation.', 'events-optimizer' );
    ?>							
								</td>
								</tr>

								<tr valign="top">					
									<th scope="row"><?php 
    esc_html_e( 'Language', 'events-optimizer' );
    ?></th>
									<td>
									<input type="text" name="openai_language" id="openai_language" size="10"  value="<?php 
    esc_attr_e( get_option( 'teo_openai_language' ) );
    ?>"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>
							

								<tr valign="top">					
								<th scope="row"><?php 
    esc_html_e( 'Tone', 'events-optimizer' );
    ?></th>
								<td>
								<select name="openai_prompt_tone" id="openai_prompt_tone" <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> >  
								<?php 
    $selected_openai_prompt_tone = urldecode( get_option( 'teo_openai_prompt_tone' ) );
    $prompt_tones = array(
        esc_html__( '- no specific tone -', 'events-optimizer' ),
        esc_html__( 'enthusiastic and energetic', 'events-optimizer' ),
        esc_html__( 'professional and informative', 'events-optimizer' ),
        esc_html__( 'warm and welcoming', 'events-optimizer' ),
        esc_html__( 'inspirational and motivational', 'events-optimizer' ),
        esc_html__( 'mysterious and intriguing', 'events-optimizer' ),
        esc_html__( 'elegant and sophisticated', 'events-optimizer' ),
        esc_html__( 'humorous and lighthearted', 'events-optimizer' ),
        esc_html__( 'educational and enlightening', 'events-optimizer' ),
        esc_html__( 'adventurous and exciting', 'events-optimizer' ),
        esc_html__( 'relaxing and tranquil', 'events-optimizer' )
    );
    foreach ( $prompt_tones as $prompt_tone ) {
        echo '<option value="' . esc_attr( $prompt_tone ) . '" ' . selected( $selected_openai_prompt_tone, $prompt_tone, false ) . '>' . esc_attr( $prompt_tone ) . '</option>';
    }
    ?>
								</select>
								<?php 
    esc_html_e( '(read more why using the right tone can', 'events-optimizer' );
    ?> 
								<a href="https://www.heinrich.biz/events-optimizer/settings/set-the-tone-of-ai-generated-descriptions/" target="_blank"  rel="noopener noreferrer">
								<?php 
    esc_html_e( 'impact the audience\'s perception', 'events-optimizer' );
    ?></a>)
								</td>
								</tr>
								
								<tr valign="top">					
									<th scope="row"><?php 
    esc_html_e( 'Max tokens', 'events-optimizer' );
    ?></th>
									<td><input type="text" name="openai_max_tokens" id="openai_max_tokens" size="5"  value="<?php 
    esc_attr_e( get_option( 'teo_openai_max_tokens' ) );
    ?>"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									<?php 
    esc_html_e( '(2048 tokens are about 900 words. Calculate them ', 'events-optimizer' );
    ?>
									<a href="https://platform.openai.com/tokenizer" target="_blank"  rel="noopener noreferrer">
									<?php 
    esc_html_e( 'here', 'events-optimizer' );
    ?></a>)
									</td>
								</tr>					
								<tr valign="top">
								<th scope="row"><?php 
    esc_html_e( 'Add FAQ', 'events-optimizer' );
    ?></th>
								<td>
									<input type="checkbox" name="openai_prompt_faq" id="openai_prompt_faq" value="true" <?php 
    checked( get_option( 'teo_openai_prompt_faq' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
										<?php 
    esc_html_e( '(improve your website user experience and increase your search engine rankings)', 'events-optimizer' );
    ?>
								</td>
								</tr>
								<tr valign="top">
								<th scope="row"><?php 
    esc_html_e( 'Add headings', 'events-optimizer' );
    ?></th>
								<td>
									<input type="checkbox" name="openai_prompt_headings" id="openai_prompt_headings" value="true" <?php 
    checked( get_option( 'teo_openai_prompt_headings' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
										<?php 
    esc_html_e( '(works only when "Output as HTML" is activated!)', 'events-optimizer' );
    ?>
								</td>
								</tr>
								<tr valign="top">
								<th scope="row"><?php 
    esc_html_e( 'Output as HTML', 'events-optimizer' );
    ?></th>
								<td>
									<input type="checkbox" name="openai_prompt_html" id="openai_prompt_html" value="true" <?php 
    checked( get_option( 'teo_openai_prompt_html' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
										<?php 
    esc_html_e( '(required if FAQ or headings added. Google Gemini generates no HTML sometimes, but Markup instead!)', 'events-optimizer' );
    ?>
								</td>
								</tr>  
								<tr valign="top">
								<th scope="row"><?php 
    esc_html_e( 'Use the content of the event, venue or organizer website for creating better descriptions', 'events-optimizer' );
    ?></th>
								<td>
									<input type="checkbox" name="openai_prompt_use_website" id="openai_prompt_use_website" value="true" <?php 
    checked( get_option( 'teo_openai_prompt_use_website' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
										<?php 
    esc_html_e( '(requires the DataForSEO API. Details see below)', 'events-optimizer' );
    ?>
								</td>
								</tr>									                                                  
								<tr valign="top">
								<th scope="row"><?php 
    esc_html_e( 'Replace existing content', 'events-optimizer' );
    ?></th>
								<td>
									<input type="checkbox" name="openai_replace_content" id="openai_replace_content" value="true" <?php 
    checked( get_option( 'teo_openai_replace_content' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
										<?php 
    esc_html_e( '(replace existing content or add new content to it)', 'events-optimizer' );
    ?>
								</td>
								</tr>
								<tr valign="top">
								<th scope="row"><?php 
    esc_html_e( 'Hide Generate description buttons', 'events-optimizer' );
    ?></th>
								<td>
									<input type="checkbox" name="openai_hide_generate_description_buttons" id="openai_hide_generate_description_buttons" value="true" <?php 
    checked( get_option( 'teo_openai_hide_generate_description_buttons' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
										<?php 
    esc_html_e( '(hide buttons in the single event, venue and organizer form)', 'events-optimizer' );
    ?>
								</td>
								</tr>							
							</table>									
							<br>

							<br><h3><?php 
    esc_html_e( 'Security', 'events-optimizer' );
    ?></h3>				
							<div id="modern-tribe-info">
								<?php 
    echo esc_html__( 'To validate event or venue websites for potential risks like malware, social engineering, etc. the free Google Safe Browsing API is used', 'events-optimizer' ) . '.<br><br>';
    echo '<li> ' . esc_html__( 'Activate the free Google SAFE BROWSING API for the relevant project in the', 'events-optimizer' ) . ' <a href="https://console.cloud.google.com/marketplace/product/google/safebrowsing.googleapis.com" target="_blank"  rel="noopener noreferrer">' . esc_html__( 'Google Cloud Console', 'events-optimizer' ) . '</a>.</li>';
    echo '<li> ' . esc_html__( 'Check out the', 'events-optimizer' ) . ' <a href="https://www.heinrich.biz/events-optimizer/settings/google-cloud-console-setup/" target="_blank" rel="noopener noreferrer">' . esc_html__( 'tutorial', 'events-optimizer' ) . '</a> ' . esc_html__( 'to find out how.', 'events-optimizer' );
    echo ' ' . esc_html__( 'Read more about', 'events-optimizer' ) . ' <a href="https://developers.google.com/safe-browsing" target="_blank" rel="noopener noreferrer">' . esc_html__( 'Google Safe Browsing', 'events-optimizer' ) . '</a> ' . esc_html__( 'or', 'events-optimizer' ) . ' <a href="https://transparencyreport.google.com/safe-browsing/search?url=https://testsafebrowsing.appspot.com/s/phishing.html" target="_blank" rel="noopener noreferrer"> ' . esc_html__( 'test it here', 'events-optimizer' ) . '</a>.</li>';
    ?>
							</div>
							<table class="form-table">	
									<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Verify URLs for potential risks', 'events-optimizer' );
    ?></th>
									<td>
										<input type="checkbox" name="google_safe_browsing" id="google_safe_browsing" value="true" <?php 
    checked( get_option( 'teo_google_safe_browsing' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
											<?php 
    esc_html_e( '(recommended to turn on)', 'events-optimizer' );
    ?>
									</td>
									</tr>
									<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Verify URLs on missing https/ http', 'events-optimizer' );
    ?></th>
									<td>
										<input type="checkbox" name="validate_url_https" id="validate_url_https" value="true" <?php 
    checked( get_option( 'teo_validate_url_https' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
											<?php 
    esc_html_e( '(recommended to turn on)', 'events-optimizer' );
    ?>
									</td>
									</tr>	
									<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Verify if URLs are redirected or permanently moved', 'events-optimizer' );
    ?></th>
									<td>
										<input type="checkbox" name="validate_url_redirected_moved" id="validate_url_redirected_moved" value="true" <?php 
    checked( get_option( 'teo_validate_url_redirected_moved' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
											<?php 
    esc_html_e( '(recommended to turn on)', 'events-optimizer' );
    ?>
									</td>
									</tr>														
							</table>

							<br><h3><?php 
    esc_html_e( 'Google Places', 'events-optimizer' );
    ?></h3>
							<div id="modern-tribe-info">

							<?php 
    echo esc_html__( 'To automatically validate and optimize venue and organizer data the Google Places API is used.', 'events-optimizer' ) . "<br><br>";
    echo '<li>' . esc_html__( 'Activate the Google PLACES API for the relevant project in the ', 'events-optimizer' ) . '<a href="https://console.cloud.google.com/marketplace/product/google/places-backend.googleapis.com" target="_blank"  rel="noopener noreferrer">' . esc_html__( 'Google Cloud Console', 'events-optimizer' ) . '</a>.</li>';
    echo '<li> ' . esc_html__( 'Check out the', 'events-optimizer' ) . ' <a href="https://www.heinrich.biz/events-optimizer/settings/google-cloud-console-setup/" target="_blank" rel="noopener noreferrer">' . esc_html__( 'tutorial', 'events-optimizer' ) . '</a> ' . esc_html__( 'to find out how', 'events-optimizer' ) . '.';
    echo ' ' . esc_html__( 'Find more information in the', 'events-optimizer' ) . ' <a href="https://developers.google.com/maps/documentation/places/web-service" target="_blank" rel="noopener noreferrer">' . esc_html__( 'Google Places API documentation', 'events-optimizer' ) . '</a>.</li>';
    echo '<li>' . esc_html__( 'Before using it please review the', 'events-optimizer' ) . ' <a href="https://developers.google.com/maps/documentation/places/web-service/usage-and-billing" target="_blank" rel="noopener noreferrer">';
    echo esc_html__( 'pricing of the Google Places API', 'events-optimizer' ) . '</a>.</li>';
    ?>		
							</div>	
																
							<table class="form-table">
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Language (ISO code)', 'events-optimizer' );
    ?></th>
									<td>
										<input type="text" name="google_places_language" id="google_places_language" size="5" value="<?php 
    esc_attr_e( get_option( 'teo_google_places_language' ) );
    ?>" <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
										<?php 
    esc_html_e( '(select from this list of the', 'events-optimizer' );
    ?> 
										<a href="https://developers.google.com/maps/faq#languagesupport" target="_blank" rel="noopener noreferrer">
										<?php 
    esc_html_e( 'Google Places API', 'events-optimizer' );
    ?></a>)
									</td>
								</tr>

								<tr valign="top">
								<th scope="row"><?php 
    esc_html_e( 'Phone number format', 'events-optimizer' );
    ?></th>
								<td>
								<select name="phone_number_format" id="phone_number_format"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> >
									<option value="international_phone_number" <?php 
    echo ( $phone_number_format === "international_phone_number" ? 'selected="selected"' : '' );
    ?>><?php 
    esc_html_e( 'International', 'events-optimizer' );
    ?></option>
									<option value="formatted_phone_number" <?php 
    echo ( $phone_number_format === "formatted_phone_number" ? 'selected="selected"' : '' );
    ?>><?php 
    esc_html_e( 'Local', 'events-optimizer' );
    ?></option>
								</select>
								<?php 
    esc_html_e( '(int. = +49..., local = (089) 1234...)', 'events-optimizer' );
    ?>
								</td>
								</tr>

								<tr valign="top">
								<th scope="row"><?php 
    esc_html_e( 'Address format', 'events-optimizer' );
    ?></th>
								<td>
								<select name="address_format" id="address_format"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> >
									<option value="street_number" <?php 
    echo ( $address_format === "street_number" ? 'selected="selected"' : '' );
    ?>><?php 
    esc_html_e( 'Street name + number', 'events-optimizer' );
    ?></option>
									<option value="number_street" <?php 
    echo ( $address_format === "number_street" ? 'selected="selected"' : '' );
    ?>><?php 
    esc_html_e( 'Street number + name', 'events-optimizer' );
    ?></option>
								</select>
								<?php 
    esc_html_e( '(Sesame street 42 vs. 42th Sesame street)', 'events-optimizer' );
    ?>
								</td>
								</tr>

								<tr valign="top">
								<th scope="row"><?php 
    esc_html_e( 'Preselection', 'events-optimizer' );
    ?></th>
								<td>
								<select name="preselection" id="preselection"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> >
									<option value="auto_completion" <?php 
    echo ( $preselection === "auto_completion" ? 'selected="selected"' : '' );
    ?>><?php 
    esc_html_e( 'Auto completion of missing data', 'events-optimizer' );
    ?></option>
									<option value="full_replacement" <?php 
    echo ( $preselection === "full_replacement" ? 'selected="selected"' : '' );
    ?>><?php 
    esc_html_e( 'Full replacement via Google Places', 'events-optimizer' );
    ?></option>
								</select>
								</td>
								</tr>
							</table>

							<br><h3><?php 
    esc_html_e( 'Fetch data from Google Places within single organizer form', 'events-optimizer' );
    ?></h3>				
							<div id="modern-tribe-info">
								<strong><?php 
    esc_html_e( 'For experienced users who are familiar with ', 'events-optimizer' );
    ?>
								<a href="<?php 
    echo esc_url( 'https://www.hostinger.com/tutorials/wordpress-custom-fields' );
    ?>" target="_blank" rel="noopener noreferrer">Custom Fields</a></strong><br><br>
								<?php 
    esc_html_e( 'In the single organizer form data from', 'events-optimizer' );
    ?> 
								<a href="<?php 
    echo esc_url( 'https://developers.google.com/maps/documentation/places/web-service/details#Place' );
    ?>" target="_blank" rel="noopener noreferrer">
								<?php 
    esc_html_e( 'Google Places', 'events-optimizer' );
    ?></a> 
								<?php 
    esc_html_e( 'can be fetched.', 'events-optimizer' );
    ?><br>
								<li><?php 
    esc_html_e( 'If you add custom fields with the following ids to the single organizer form they will be populated with the corresponding data from Google Places:', 'events-optimizer' );
    ?><br>		
								<?php 
    esc_html_e( 'OrganizerAddress, OrganizerCity, OrganizerZip, OrganizerProvince, OrganizerState, OrganizerCountry, OrganizerCurrentOpeningHours, OrganizerRating', 'events-optimizer' );
    ?></li>												
								<li><?php 
    esc_html_e( 'The full address can be populated in any input or textarea field. Enter the field id below.', 'events-optimizer' );
    ?></li>				
							</div>

							<table class="form-table">	
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Field id for full address', 'events-optimizer' );
    ?></th>
									<td>
										<input type="text" name="element_id" id="element_id" size="30" value="<?php 
    echo esc_attr_e( get_option( 'teo_element_id' ) );
    ?>"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
										<?php 
    esc_html_e( '(e.g. content or excerpt)', 'events-optimizer' );
    ?>
									</td>
								</tr>
							</table>

							<br><h3><?php 
    esc_html_e( 'Google Images', 'events-optimizer' );
    ?></h3>
							<div id="modern-tribe-info">
								<li><?php 
    esc_html_e( 'To automatically add Google Images the DataForSEO API is used.', 'events-optimizer' );
    ?></li>
								<li><?php 
    esc_html_e( 'Use it responsibly, respecting copyright laws to avoid legal issues.', 'events-optimizer' );
    ?></li>
							</div>
																
							<table class="form-table">
								<tr valign="top">					
									<th scope="row"><?php 
    esc_html_e( 'Maximum images to search', 'events-optimizer' );
    ?></th>
									<td>
										<input type="text" name="max_images" id="max_images" size="2"  value="<?php 
    esc_attr_e( get_option( 'teo_max_images' ) );
    ?>"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>	
							</table>								
							<br>

							<br><h3><?php 
    esc_html_e( 'Google Events', 'events-optimizer' );
    ?><span class='beta_version'>BETA</span></h3>
							<div id="modern-tribe-info">
								<li><?php 
    esc_html_e( 'To automatically add Google Events the DataForSEO API is used.', 'events-optimizer' );
    ?></li>
								<li><?php 
    esc_html_e( 'Test the search results at', 'events-optimizer' );
    ?> 
								<a href="https://www.google.com/search?q=event&ibp=htl;events&rciv=evn#fpstate=tldetail" target="_blank" rel="noopener noreferrer">
								<?php 
    esc_html_e( 'Google Events', 'events-optimizer' );
    ?></a>.</li>
								<li><?php 
    esc_html_e( 'Regularly adding events, such as weekly or monthly, can help prevent duplicates', 'events-optimizer' );
    ?>.</li>
							</div>
																
							<table class="form-table">						
								<tr valign="top">							
									<th scope="row"><?php 
    esc_html_e( 'Date range', 'events-optimizer' );
    ?></th>
									<td>
										<select name="dataforseo_date_range" id="dataforseo_date_range" <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?>>
										<?php 
    $date_range_options = array(
        'all'        => esc_html__( 'All upcoming events', 'events-optimizer' ),
        'today'      => esc_html__( 'Today', 'events-optimizer' ),
        'tomorrow'   => esc_html__( 'Tomorrow', 'events-optimizer' ),
        'weekend'    => esc_html__( 'This weekend', 'events-optimizer' ),
        'week'       => esc_html__( 'This week', 'events-optimizer' ),
        'next_week'  => esc_html__( 'Next week', 'events-optimizer' ),
        'month'      => esc_html__( 'This month', 'events-optimizer' ),
        'next_month' => esc_html__( 'Next month', 'events-optimizer' ),
    );
    foreach ( $date_range_options as $dr_value => $dr_decription ) {
        echo '<option value="' . esc_attr( $dr_value ) . '"' . selected( $dr_value, get_option( 'teo_dataforseo_date_range' ), false ) . '>' . esc_attr( $dr_decription ) . '</option>';
    }
    ?>
										</select>  
									</td>
								</tr>
								<tr valign="top">							
									<th scope="row"><?php 
    esc_html_e( 'Language (ISO code)', 'events-optimizer' );
    ?></th>
									<td>
										<input type="text" name="dataforseo_language_code" id="dataforseo_language_code" size="5"  value="<?php 
    esc_attr_e( get_option( 'teo_dataforseo_language_code' ) );
    ?>"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />	
										<?php 
    esc_html_e( '(select from', 'events-optimizer' );
    ?> 
										<a href="https://cdn.dataforseo.com/v3/languages/languages_serp_google_2022_01_17.csv">
										<?php 
    esc_html_e( 'this list from DataForSEO', 'events-optimizer' );
    ?></a>)
									</td>
								</tr>							
								<tr valign="top">							
									<th scope="row"><?php 
    esc_html_e( 'Location code', 'events-optimizer' );
    ?></th>
									<td>
										<input type="text" name="dataforseo_location_code" id="dataforseo_location_code" size="10"  value="<?php 
    esc_attr_e( get_option( 'teo_dataforseo_location_code' ) );
    ?>"  <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
										<?php 
    esc_html_e( '(select from', 'events-optimizer' );
    ?>
										<a href="https://cdn.dataforseo.com/v3/locations/locations_serp_google_2023_11_17.csv">
										<?php 
    esc_html_e( 'this list from DataForSEO', 'events-optimizer' );
    ?></a>)
									</td>
								</tr>
							</table>							
							<table class="form-table">
								<div id="modern-tribe-info">
									<?php 
    esc_html_e( 'The following events are already added from Google Events. They will be ignored when searching for additional events.', 'events-optimizer' );
    ?><br>
								</div>			
								<tr valign="top">
								<th scope="row">
									<input type="button" name="action" class="button" id="reset_objects_added_button" value="<?php 
    esc_html_e( 'Reset list', 'events-optimizer' );
    ?>" <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
								</th>
								<td>
								<?php 
    echo '<p><strong>' . esc_html__( 'Google Events', 'events-optimizer' ) . '</strong></p>';
    $ignored_and_added_objects = maybe_unserialize( get_option( 'teo_ignored_and_added_objects' ) );
    if ( is_array( $ignored_and_added_objects ) ) {
        $events = array_filter( $ignored_and_added_objects, function ( $object ) {
            return isset( $object['status'] ) && $object['status'] === 'added';
        } );
        if ( empty( $events ) ) {
            echo '--<br>';
        } else {
            foreach ( $events as $event ) {
                $object_id = esc_attr( $event['id'] );
                $post_id = esc_attr( $event['post_id'] );
                $start_datetime_gp = esc_attr( $event['start_datetime_gp'] );
                $post_title = esc_html( get_the_title( $post_id ) );
                $edit_url = esc_url( "/wp-admin/post.php?post={$post_id}&action=edit" );
                echo "<div id='object_{$object_id}'>{$post_title} (" . date_format( date_create( $start_datetime_gp ), 'm.d.Y, H:i' ) . ")</div>";
            }
        }
    } else {
        echo '--<br>';
    }
    ?>
								</td>							
								</tr>						
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Show added events', 'events-optimizer' );
    ?></th>
									<td>
										<input type="checkbox" name="show_events_added" id="show_events_added" value="true" <?php 
    checked( get_option( 'teo_show_events_added' ), 'true' );
    ?> <?php 
    disabled( teo_fs()->can_use_premium_code__premium_only(), false );
    ?> />
									</td>
								</tr>																																																					
							</table>
						

							<br><h3><?php 
    esc_html_e( 'Ignored objects', 'events-optimizer' );
    ?></h3>				
							<div id="modern-tribe-info">
								<?php 
    esc_html_e( 'The following objects are ignored when searching for issues.', 'events-optimizer' );
    ?><br>
							</div>
							<table class="form-table">	
								<tr valign="top">
								<th scope="row">
									<input type="button" name="action" class="button" id="reset_objects_ignored_button" value="<?php 
    esc_html_e( 'Reset list', 'events-optimizer' );
    ?>" <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
								</th>
								<td>
									<div class="row">					
										<?php 
    $ignored_and_added_objects = maybe_unserialize( get_option( 'teo_ignored_and_added_objects' ) );
    echo '<p><strong>' . esc_html__( 'Venues', 'events-optimizer' ) . '</strong></p>';
    if ( is_array( $ignored_and_added_objects ) ) {
        $venues = array_filter( $ignored_and_added_objects, function ( $object ) {
            return isset( $object['status'], $object['type'] ) && $object['status'] === 'ignored' && $object['type'] === 'tribe_venue';
        } );
        if ( empty( $venues ) ) {
            echo '--<br>';
        } else {
            foreach ( $venues as $venue ) {
                $object_id = esc_attr( $venue['id'] );
                $post_id = esc_attr( $venue['post_id'] );
                $post_title = esc_html( get_the_title( $post_id ) );
                $edit_url = esc_url( "/wp-admin/post.php?post={$post_id}&action=edit" );
                echo "<div id='object_{$object_id}'>{$post_title}</div>";
            }
        }
    } else {
        echo '--<br>';
    }
    echo '</div><br>';
    echo '<p><strong>' . esc_html__( 'Organizers', 'events-optimizer' ) . '</strong></p>';
    if ( is_array( $ignored_and_added_objects ) ) {
        $organizers = array_filter( $ignored_and_added_objects, function ( $object ) {
            return $object['status'] === 'ignored' && $object['type'] === 'tribe_organizer';
        } );
        if ( empty( $organizers ) ) {
            echo '--<br>';
        } else {
            foreach ( $organizers as $organizer ) {
                $object_id = esc_attr( $organizer['id'] );
                $post_id = esc_attr( $organizer['post_id'] );
                $post_title = esc_html( stripslashes( get_the_title( $post_id ) ) );
                $edit_url = esc_url( "/wp-admin/post.php?post={$post_id}&action=edit" );
                echo "<div id='object_{$object_id}'>{$post_title}</div>";
            }
        }
    } else {
        echo '--<br>';
    }
    echo '</div><br>';
    echo '<p><strong>' . esc_html__( 'Events', 'events-optimizer' ) . '</strong></p>';
    if ( is_array( $ignored_and_added_objects ) ) {
        $events = array_filter( $ignored_and_added_objects, function ( $object ) {
            return $object['status'] === 'ignored' && $object['type'] === 'tribe_events';
        } );
        if ( empty( $events ) ) {
            echo '--<br>';
        } else {
            foreach ( $events as $event ) {
                $object_id = esc_attr( $event['id'] );
                $post_id = esc_attr( $event['post_id'] );
                $title_gp = esc_html( stripslashes( $event['title_gp'] ) );
                $post_title = esc_html( get_the_title( $post_id ) );
                $edit_url = esc_url( "/wp-admin/post.php?post={$post_id}&action=edit" );
                $start_datetime_gp = esc_html( date_format( date_create( $event['start_datetime_gp'] ), get_option( 'date_format' ) . ", H:i" ) );
                if ( $post_id !== '' ) {
                    echo "<div id='object_{$object_id}'>{$post_title}</div>";
                } else {
                    echo "<div id='object_{$object_id}'>{$title_gp} ({$start_datetime_gp})</div>";
                }
            }
        }
    } else {
        echo '--<br>';
    }
    echo '</div>';
    ?>
									</div>						
								</td>
								</tr>							
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Show ignored objects', 'events-optimizer' );
    ?></th>
									<td>
										<input type="checkbox" name="show_ignored_objects" id="show_ignored_objects" value="true" <?php 
    checked( get_option( 'teo_show_ignored_objects' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>
								</table>

								<br><h3><?php 
    esc_html_e( 'Ignored fields', 'events-optimizer' );
    ?></h3>				
								<div id="modern-tribe-info">
									<?php 
    esc_html_e( 'All fields checked will be ignored when searching for invalid data.', 'events-optimizer' );
    ?><br>				
								</div>
								
							<br><h3><?php 
    esc_html_e( 'Venues', 'events-optimizer' );
    ?></h3>
							<table class="form-table">	
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Description', 'events-optimizer' );
    ?></th>
									<td>
										<input type="checkbox" name="ignored_field_venue_description" id="ignored_field_venue_description" value="true" <?php 
    checked( get_option( 'teo_ignored_field_venue_description' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>                        
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Address', 'events-optimizer' );
    ?></th>
									<td>
										<input type="checkbox" name="ignored_field_venue_address" id="ignored_field_venue_address" value="true" <?php 
    checked( get_option( 'teo_ignored_field_venue_address' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'ZIP', 'events-optimizer' );
    ?></th>
									<td>
										<input type="checkbox" name="ignored_field_venue_zip" id="ignored_field_venue_zip" value="true" <?php 
    checked( get_option( 'teo_ignored_field_venue_zip' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'City', 'events-optimizer' );
    ?></th>
									<td>
										<input type="checkbox" name="ignored_field_venue_city" id="ignored_field_venue_city" value="true" <?php 
    checked( get_option( 'teo_ignored_field_venue_city' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Province', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_venue_province" id="ignored_field_venue_province" value="true" <?php 
    checked( get_option( 'teo_ignored_field_venue_province' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'State', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_venue_state" id="ignored_field_venue_state" value="true" <?php 
    checked( get_option( 'teo_ignored_field_venue_state' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Country', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_venue_country" id="ignored_field_venue_country" value="true" <?php 
    checked( get_option( 'teo_ignored_field_venue_country' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>  
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Phone', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_venue_phone" id="ignored_field_venue_phone" value="true" <?php 
    checked( get_option( 'teo_ignored_field_venue_phone' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>  
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Website', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_venue_website" id="ignored_field_venue_website" value="true" <?php 
    checked( get_option( 'teo_ignored_field_venue_website' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Business Status', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_venue_business_status" id="ignored_field_venue_business_status" value="true" <?php 
    checked( get_option( 'teo_ignored_field_venue_business_status' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>																											
							</table>

							<br><h3><?php 
    esc_html_e( 'Organizers', 'events-optimizer' );
    ?></h3>
							<table class="form-table">	
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Description', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_organizer_description" id="ignored_field_organizer_description" value="true" <?php 
    checked( get_option( 'teo_ignored_field_organizer_description' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>								
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Phone', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_organizer_phone" id="ignored_field_organizer_phone" value="true" <?php 
    checked( get_option( 'teo_ignored_field_organizer_phone' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>	
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Website', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_organizer_website" id="ignored_field_organizer_website" value="true" <?php 
    checked( get_option( 'teo_ignored_field_organizer_website' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>																				
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Mail', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_organizer_mail" id="ignored_field_organizer_mail" value="true" <?php 
    checked( get_option( 'teo_ignored_field_organizer_mail' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Business Status', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_organizer_business_status" id="ignored_field_organizer_business_status" value="true" <?php 
    checked( get_option( 'teo_ignored_field_organizer_business_status' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>	
							</table>

							<br><h3><?php 
    esc_html_e( 'Events', 'events-optimizer' );
    ?></h3>
							<table class="form-table">	
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Description', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_event_description" id="ignored_field_event_description" value="true" <?php 
    checked( get_option( 'teo_ignored_field_event_description' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>      
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Start', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_event_start" id="ignored_field_event_start" value="true" <?php 
    checked( get_option( 'teo_ignored_field_event_start' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>  
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'End', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_event_end" id="ignored_field_event_end" value="true" <?php 
    checked( get_option( 'teo_ignored_field_event_end' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>                              
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Venue', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_event_venue" id="ignored_field_event_venue" value="true" <?php 
    checked( get_option( 'teo_ignored_field_event_venue' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>                                         
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Organizer', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_event_organizer" id="ignored_field_event_organizer" value="true" <?php 
    checked( get_option( 'teo_ignored_field_event_organizer' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Website', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_event_website" id="ignored_field_event_website" value="true" <?php 
    checked( get_option( 'teo_ignored_field_event_website' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>  
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Currency symbol', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_event_currency_symbol" id="ignored_field_event_currency_symbol" value="true" <?php 
    checked( get_option( 'teo_ignored_field_event_currency_symbol' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>
								<tr valign="top">
									<th scope="row"><?php 
    esc_html_e( 'Cost', 'events-optimizer' );
    ?></th>
									<td><input type="checkbox" name="ignored_field_event_cost" id="ignored_field_event_cost" value="true" <?php 
    checked( get_option( 'teo_ignored_field_event_cost' ), 'true' );
    ?> <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?> />
									</td>
								</tr>
							</table>

							<br><h3><?php 
    esc_html_e( 'Permissions', 'events-optimizer' );
    ?></h3>

							<div id="modern-tribe-info">
									<?php 
    echo esc_html__( 'Only users with administrator rights can access the Events Optimizer by default. Use a plugin like the', 'events-optimizer' ) . ' <a href="https://de.wordpress.org/plugins/wp-user-manager/" target="_blank" rel="noopener noreferrer">' . esc_html__( 'WP User Manager', 'events-optimizer' ) . '</a> ' . esc_html__( 'to create an additional role. Select the role below to give it access to Events Optimizer as well.', 'events-optimizer' );
    ?>
									<br>				
							</div>					

							<table class="form-table">			
								<tr valign="top">							
									<th scope="row"><?php 
    esc_html_e( 'Role that has access to Events Optimizer', 'events-optimizer' );
    ?></th>
									<td>
										<select name="allowed_role" id="allowed_role" <?php 
    disabled( !teo_fs()->can_use_premium_code__premium_only() );
    ?>>
										<?php 
    $all_roles = wp_roles()->roles;
    $editable_roles = apply_filters( 'editable_roles', $all_roles );
    // Remove the the WordPress standard roles from the list
    unset($editable_roles['editor']);
    unset($editable_roles['author']);
    unset($editable_roles['contributor']);
    unset($editable_roles['subscriber']);
    foreach ( $editable_roles as $role => $details ) {
        $name = translate_user_role( $details['name'] );
        if ( $role === 'administrator' ) {
            $name = esc_html__( 'Only the administrator', 'events-optimizer' );
        }
        echo '<option value="' . esc_attr( $role ) . '"' . selected( $role, get_option( 'teo_allowed_role' ), false ) . '>' . esc_html( $name ) . '</option>';
    }
    ?>
										</select>
									</td>
								</tr>
							</table>	
							<br><br>
							<input type='submit' name='action' class='button-primary' id='save_settings' value='<?php 
    esc_html_e( 'Save changes', 'events-optimizer' );
    ?>' />
						</div>
					</form>
					<p>
						<div id='show_tab_settings'></div>
						<div id='loading_tab_settings'><?php 
    esc_html_e( 'SAVING', 'events-optimizer' );
    ?></div>
					</p>
				</div>
			</div>

			<!-- jQuery: Fade in/out buttons and call php functions to analyse data -->	
			<script type='text/javascript'>
			
				// Wait until the whole document has fully loaded
				jQuery(document).ready(function(){	
					
					// Show venues with invalid data
					jQuery('#venues_with_invalid_data').on('click', function()  { // Execute function when button is clicked
						jQuery("#venues_with_invalid_data").attr("disabled", true); // Disable button while analyzing
						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'venues_wid', // call function "Venues with invalid data"
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>'
							},
							beforeSend: function() {
								jQuery("#loading_tab_venues").fadeIn('fast'); // Fade in loading icon and "analyzing" text
								jQuery("#show_results_venues").fadeOut("fast"); // Hide results otherwise they are show on the other tabs!
							},
							success: function(html){ // Function is called and if data is retrieved, store it in variable "html"
								jQuery("#loading_tab_venues").fadeOut('slow'); // Fade out loading icon and "analyzing" text
								jQuery("#show_results_venues").html( html ); // Show the html inside "show_results_venues" div
								jQuery("#show_results_venues").fadeIn("fast"); // Fade in the results
								jQuery("#venues_with_invalid_data").attr("disabled", false); // Enable button after analyzing
							}
						}); //close jQuery.ajax
						return false;
					});	

					// Show duplicate venues
					jQuery('#duplicate_venues').on('click', function()  {
						jQuery("#duplicate_venues").attr("disabled", true);		
						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'duplicate_ven', // call function "Duplicate Venues"
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>'
							},
							beforeSend: function() {
								jQuery("#loading_tab_venues").fadeIn('fast');
								jQuery("#show_results_venues").fadeOut("fast");
							},
							success: function(html){
								jQuery("#loading_tab_venues").fadeOut('slow');
								jQuery("#show_results_venues").html( html );
								jQuery("#show_results_venues").fadeIn("fast");
								jQuery("#duplicate_venues").attr("disabled", false);
							}
						});
						return false;					
					});
					
					// Optimize venues with Google Places data
					jQuery('#venues_google_places_optimization').on('click', function()  {
						jQuery("#venues_google_places_optimization").attr("disabled", true);			
						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'venues_optimization_wgp', // call function "Venues optimization with Google Places"
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>'
							},
							beforeSend: function() {
								jQuery("#loading_tab_venues").fadeIn('fast');
								jQuery("#show_results_venues").fadeOut("fast");
							},
							success: function(html){
								jQuery("#loading_tab_venues").fadeOut('slow');
								jQuery("#show_results_venues").html( html );
								jQuery("#show_results_venues").fadeIn("fast");
								jQuery("#venues_google_places_optimization").attr("disabled", false);						
							}
						});
						return false;					
					});
					
					// Show venues with inactive business operation
					jQuery('#venues_with_inactive_business_operation').on('click', function()  {
						jQuery("#venues_with_inactive_business_operation").attr("disabled", true);	
						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'venues_with_inactive_bo', // call function "Venues with inactive business operation"
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>'
							},
							beforeSend: function() {
								jQuery("#loading_tab_venues").fadeIn('fast');
								jQuery("#show_results_venues").fadeOut("fast");
							},
							success: function(html){
								jQuery("#loading_tab_venues").fadeOut('slow');
								jQuery("#show_results_venues").html( html );
								jQuery("#show_results_venues").fadeIn("fast");
								jQuery("#venues_with_inactive_business_operation").attr("disabled", false);						
							}
						});
						return false;					
					});

					// Process the bulk actions	for venues with inactive business operation	
					jQuery(document).on('click', '#doaction, #doaction2', function() {	
						var bulk_action_top = jQuery('#bulk-action-selector-top').val();
						var bulk_action_bottom = jQuery('#bulk-action-selector-bottom').val();
						var bulk_action = bulk_action_top !== '-1' ? bulk_action_top : bulk_action_bottom;

						var ids = [];
						jQuery('.check-column input:checked').each(function() {
							ids.push(jQuery(this).val());
						});	

						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'venues_with_inactive_bo', // call function "Venues with inactive business operation"
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>',
								bulk_action: bulk_action,
								ids: ids
							},
							beforeSend: function() {
							},
							success: function(html){
								jQuery("#show_results_venues").html( html );					
							}
						});
						return false;					
					});

					// Navigate through pages in venues with inactive business operation
					jQuery(document).on('click', '.tablenav-pages a', function() {
						var pageHref = jQuery(this).attr('href');
						var pageNumber = pageHref.split('paged=')[1]; // Extract page number from URL

						if (!pageNumber) pageNumber = 1; // Set pageNumber to 1 if page number is not found

						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'venues_with_inactive_bo', // call function "Venues with inactive business operation"
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>',
								paged: pageNumber
							},
							beforeSend: function() {
							},
							success: function(html){
								jQuery("#show_results_venues").html( html );
							}
						});
						return false;					
					});

					// Show organizers with invalid data
					jQuery('#organizers_with_invalid_data').on('click', function()  {
						jQuery("#organizers_with_invalid_data").attr("disabled", true);
						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'organizers_wid', // call function "Organizers with invalid data"
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>'
							},
							beforeSend: function() {
								jQuery("#loading_tab_organizers").fadeIn('fast');
								jQuery("#show_results_organizers").fadeOut("fast");
							},
							success: function(html){
								jQuery("#loading_tab_organizers").fadeOut('slow');
								jQuery("#show_results_organizers").html( html );
								jQuery("#show_results_organizers").fadeIn("fast");
								jQuery("#organizers_with_invalid_data").attr("disabled", false);
							}
						});
						return false;
					});	
		
					// Show duplicate organizers
					jQuery('#duplicate_organizers').on('click', function()  {
						jQuery("#duplicate_organizers").attr("disabled", true);		
						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'duplicate_org',  // call function "Duplicate Organizers"
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>' 
							},
							beforeSend: function() {
								jQuery("#loading_tab_organizers").fadeIn('fast');
								jQuery("#show_results_organizers").fadeOut("fast");
							},
							success: function(html){
								jQuery("#loading_tab_organizers").fadeOut('slow');
								jQuery("#show_results_organizers").html( html );
								jQuery("#show_results_organizers").fadeIn("fast");
								jQuery("#duplicate_organizers").attr("disabled", false);
							}
						});
						return false;					
					});	

					// Optimize organizers with Google Places data
					jQuery('#organizers_google_places_optimization').on('click', function()  {
						jQuery("#organizers_google_places_optimization").attr("disabled", true);
						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'organizers_optimization_wgp', // call function "Organizers optimization with Google Places"
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>' 
							},
							beforeSend: function() {
								jQuery("#loading_tab_organizers").fadeIn('fast');
								jQuery("#show_results_organizers").fadeOut("fast");
							},
							success: function(html){
								jQuery("#loading_tab_organizers").fadeOut('slow');
								jQuery("#show_results_organizers").html( html );
								jQuery("#show_results_organizers").fadeIn("fast");
								jQuery("#organizers_google_places_optimization").attr("disabled", false);
							}
						});
						return false;					
					})		

					// Show events with invalid data
					jQuery('#events_with_invalid_data').on('click', function()  {
						jQuery("#events_with_invalid_data").attr("disabled", true);		
						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'events_wid', // call function "Events with invalid data"
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>' 
							},
							beforeSend: function() {
								jQuery("#loading_tab_events").fadeIn('fast');
								jQuery("#show_results_events").fadeOut("fast");
							},
							success: function(html){
								jQuery("#loading_tab_events").fadeOut('slow');
								jQuery("#show_results_events").html( html );
								jQuery("#show_results_events").fadeIn("fast");
								jQuery("#events_with_invalid_data").attr("disabled", false);
							}
						});
						return false;					
					});	

					// Search events with Google data
					jQuery('#search_events_with_google_data').on('click', function()  {
						jQuery("#search_events_with_google_data").attr("disabled", true);
						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'search_events_wgd', // call function "Search events with Google data"
								dataforseo_keyword:	jQuery( '#dataforseo_keyword' ).val(),
								dataforseo_depth:	jQuery( '#dataforseo_depth' ).val(),							
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>' 
							},
							beforeSend: function() {
								//jQuery("#loading_tab_events").text('ANALYZING');							
								jQuery("#loading_tab_events").fadeIn('fast');
								jQuery("#show_results_events").fadeOut("fast");
							},
							success: function(html){
								jQuery("#loading_tab_events").fadeOut('slow');
								jQuery("#show_results_events").html( html );
								jQuery("#show_results_events").fadeIn("fast");
								jQuery("#search_events_with_google_data").attr("disabled", false);
							}
						});
						return false;					
					});	
					
					// Add events with Google data
					jQuery('#add_events_with_google_data').on('click', function()  {
						jQuery("#add_events_with_google_data").attr("disabled", true);		
						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'add_events_wgd', // call function "Add events from Google data"
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>' 
							},
							beforeSend: function() {
								//jQuery("#loading_tab_events").text('DOWNLOAD SEARCH RESULTS');						
								jQuery("#loading_tab_events").fadeIn('fast');
								jQuery("#show_results_events").fadeOut("fast");
							},
							success: function(html){
								jQuery("#loading_tab_events").fadeOut('slow');
								jQuery("#show_results_events").html( html );
								jQuery("#show_results_events").fadeIn("fast");
								jQuery("#add_events_with_google_data").attr("disabled", false);
							}
						});
						return false;					
					});	

					// Save settings
					jQuery('#save_settings').on('click', function()  { //start function when button is clicked
						
						jQuery("#save_settings").attr("disabled", true); // Disable button while saving
						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: {
								action: 'save_set', // call function "Save settings"
								max_results_per_page: 					jQuery( '#max_results_per_page' ).val(),

								duplicate_organizers_levenshtein: 		jQuery( '#duplicate_organizers_levenshtein' ).val(),
								duplicate_venues_levenshtein: 			jQuery( '#duplicate_venues_levenshtein' ).val(),

								google_places_api_key: 					jQuery( '#google_places_api_key' ).val(),
								google_places_language: 				jQuery( '#google_places_language' ).val(),
								
								dataforseo_keyword:						jQuery( '#dataforseo_keyword' ).val(),
								dataforseo_api_login:					jQuery( '#dataforseo_api_login' ).val(),
								dataforseo_api_password:				jQuery( '#dataforseo_api_password' ).val(),
								dataforseo_location_code:				jQuery( '#dataforseo_location_code' ).val(),
								dataforseo_language_code:				jQuery( '#dataforseo_language_code' ).val(),							
								dataforseo_date_range:					jQuery( '#dataforseo_date_range' ).val(),							
								max_images:								jQuery( '#max_images' ).val(),		
							
								openai_api_key: 						jQuery( '#openai_api_key' ).val(),
								openai_model:							jQuery( '#openai_model' ).val(),							
								openai_max_tokens: 						jQuery( '#openai_max_tokens' ).val(),									
								openai_language: 						jQuery( '#openai_language' ).val() ,
								openai_prompt_tone: 					jQuery( '#openai_prompt_tone' ).val() ,							
								openai_prompt_headings: 				document.getElementById( 'openai_prompt_headings' ).checked,										
								openai_prompt_faq: 						document.getElementById( 'openai_prompt_faq' ).checked,	
								openai_prompt_html: 					document.getElementById( 'openai_prompt_html' ).checked,
								openai_prompt_use_website:				document.getElementById( 'openai_prompt_use_website' ).checked,
								openai_replace_content: 				document.getElementById( 'openai_replace_content' ).checked,	
								openai_hide_generate_description_buttons:	document.getElementById( 'openai_hide_generate_description_buttons' ).checked,	

								gemini_api_key: 						jQuery( '#gemini_api_key' ).val(),
								gemini_model:							jQuery( '#gemini_model' ).val(),

								google_safe_browsing: 					document.getElementById( 'google_safe_browsing' ).checked,
								validate_url_https: 					document.getElementById( 'validate_url_https' ).checked,
								validate_url_redirected_moved: 			document.getElementById( 'validate_url_redirected_moved' ).checked,							

								phone_number_format: 					jQuery( '#phone_number_format' ).val(),
								address_format: 						jQuery( '#address_format' ).val(),		
								preselection: 							jQuery( '#preselection' ).val(),

								element_id: 							jQuery( '#element_id' ).val(),
								show_ignored_objects: 					document.getElementById( 'show_ignored_objects' ).checked,	
								show_events_added: 						document.getElementById( 'show_events_added' ).checked,
								
								ignored_field_venue_description: 		document.getElementById( 'ignored_field_venue_description' ).checked,			
								ignored_field_venue_address: 			document.getElementById( 'ignored_field_venue_address' ).checked,								
								ignored_field_venue_zip: 				document.getElementById( 'ignored_field_venue_zip' ).checked,								
								ignored_field_venue_city: 				document.getElementById( 'ignored_field_venue_city' ).checked,								
								ignored_field_venue_province: 			document.getElementById( 'ignored_field_venue_province' ).checked,								
								ignored_field_venue_state: 				document.getElementById( 'ignored_field_venue_state' ).checked,								
								ignored_field_venue_country: 			document.getElementById( 'ignored_field_venue_country' ).checked,								
								ignored_field_venue_phone: 				document.getElementById( 'ignored_field_venue_phone' ).checked,								
								ignored_field_venue_website: 			document.getElementById( 'ignored_field_venue_website' ).checked,	
								ignored_field_venue_business_status: 	document.getElementById( 'ignored_field_venue_business_status' ).checked,
																						
								ignored_field_organizer_description: 	document.getElementById( 'ignored_field_organizer_description' ).checked,								
								ignored_field_organizer_phone: 			document.getElementById( 'ignored_field_organizer_phone' ).checked,								
								ignored_field_organizer_website: 		document.getElementById( 'ignored_field_organizer_website' ).checked,								
								ignored_field_organizer_mail: 			document.getElementById( 'ignored_field_organizer_mail' ).checked,
								ignored_field_organizer_business_status:document.getElementById( 'ignored_field_organizer_business_status' ).checked,
																							
								ignored_field_event_description: 		document.getElementById( 'ignored_field_event_description' ).checked,								
								ignored_field_event_start: 				document.getElementById( 'ignored_field_event_start' ).checked,								
								ignored_field_event_end: 				document.getElementById( 'ignored_field_event_end' ).checked,								
								ignored_field_event_venue: 				document.getElementById( 'ignored_field_event_venue' ).checked,								
								ignored_field_event_organizer: 			document.getElementById( 'ignored_field_event_organizer' ).checked,
								ignored_field_event_website: 			document.getElementById( 'ignored_field_event_website' ).checked,								
								ignored_field_event_currency_symbol: 	document.getElementById( 'ignored_field_event_currency_symbol' ).checked,		
								ignored_field_event_cost: 				document.getElementById( 'ignored_field_event_cost' ).checked,								

								allowed_role: 							jQuery( '#allowed_role' ).val(),							
						
								//remove_settings: 						jQuery( '#remove_settings' ).val(),																							
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>'																		
							},
							beforeSend: function() {
								jQuery("#loading_tab_settings").fadeIn('fast');
								jQuery("#show_tab_settings").fadeOut("slow");
							}, 
							success: function(html){ //if data is retrieved, store it in html
								jQuery("#loading_tab_settings").fadeOut('slow');
								jQuery("#show_tab_settings").html( html ); //show the html inside formstatus div
								jQuery("#show_tab_settings").fadeIn("fast"); //animation
								jQuery("#save_settings").attr("disabled", false);
								location.reload(); // reload setting page that everything is shown with updated data
							}
						}); //close jQuery.ajax
						return false;					
					});

					// Reset list with objects added
					jQuery('#reset_objects_added_button').on('click', function()  {
						jQuery("#reset_objects_added_button").attr("disabled", true);	
						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'reset_object_li', // call function "reset objects list"
								status: 'added',
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>' 
							},
							beforeSend: function() {
							},
							success: function(html){
								jQuery("#reset_objects_added_button").attr("disabled", false);
								location.reload(); // reload setting page that everything is shown with updated data						
							}
						});
						return false;					
					});	

					// Reset list with objects ignored
					jQuery('#reset_objects_ignored_button').on('click', function()  {
						jQuery("#reset_objects_ignored_button").attr("disabled", true);	
						jQuery.ajax({
							type: "POST",
							url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
							data: { 
								action: 'reset_object_li', // call function "reset objects list"
								status: 'ignored',
								_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>' 
							},
							beforeSend: function() {
							},
							success: function(html){
								jQuery("#reset_objects_ignored_button").attr("disabled", false);
								location.reload(); // reload setting page that everything is shown with updated data						
							}
						});
						return false;					
					});	
																				
				});
				
				// Remove object from list
				function remove_object(id) {
					jQuery("#object_"+id).attr('disabled','disabled').css('opacity',0.4);
					jQuery.ajax({
						type: "POST",
						url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
						data: { 
							action: 'remove_obj',
							_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>',
							id: id,
						},
						beforeSend: function() {
							jQuery("#object_"+id).fadeOut('fast');	// Fade out status after data is ignored	
						},
						success: function(html) {					
						}
					}); //close jQuery.ajax
					return false;	
				}			

			</script>
			
		<?php 
}

//----------------------------------------------------------------------------------------
// Venues with invalid data
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_venues_wid', 'venues_with_invalid_data' );
function venues_with_invalid_data() {
    // Check the nonce
    if ( !check_ajax_referer( 'events-optimizer-nonce', '_wpnonce', false ) ) {
        wp_die( 'Invalid nonce' );
    }
    // Maximum number of results to display per page
    $max_results_per_page = intval( get_option( 'teo_max_results_per_page', 5 ) );
    // Objects to be ignored while searching for invalid data
    $ignored_and_added_objects = maybe_unserialize( get_option( 'teo_ignored_and_added_objects' ) );
    if ( empty( $ignored_and_added_objects ) ) {
        $ignored_and_added_objects = array();
    } else {
        $ignored_and_added_objects = sanitize_multi_dimensional_array( $ignored_and_added_objects );
    }
    // Whether ignored objects should be shown or not
    $show_ignored_objects = sanitize_text_field( get_option( 'teo_show_ignored_objects' ) );
    // Whether added objects should be shown or not
    $show_added_objects = sanitize_text_field( get_option( 'teo_show_added_objects' ) );
    // Fields to be ignored while searching for invalid data
    $ignored_field_venue_description = get_option( 'teo_ignored_field_venue_description' );
    $ignored_field_venue_address = get_option( 'teo_ignored_field_venue_address' );
    $ignored_field_venue_zip = get_option( 'teo_ignored_field_venue_zip' );
    $ignored_field_venue_city = get_option( 'teo_ignored_field_venue_city' );
    $ignored_field_venue_province = get_option( 'teo_ignored_field_venue_province' );
    $ignored_field_venue_state = get_option( 'teo_ignored_field_venue_state' );
    $ignored_field_venue_country = get_option( 'teo_ignored_field_venue_country' );
    $ignored_field_venue_phone = get_option( 'teo_ignored_field_venue_phone' );
    $ignored_field_venue_website = get_option( 'teo_ignored_field_venue_website' );
    // Custom countries from Events Calendar Options
    $options = get_option( 'tribe_events_calendar_options', '' );
    if ( !empty( $options['tribe_events_calendar_options'] ) ) {
        $countries = explode( PHP_EOL, $options['tribeEventsCountries'] );
    } else {
        $countries = array();
    }
    foreach ( $countries as $j => $country ) {
        $country = sanitize_text_field( $country );
        if ( !empty( trim( substr( $country, strpos( $country, ' ' ) + 1 ) ) ) ) {
            $countries_arr[$j] = trim( substr( $country, strpos( $country, ' ' ) + 1 ) );
        }
    }
    $fields_arr = array(
        'ID'             => esc_html__( 'ID', 'events-optimizer' ),
        'post_title'     => esc_html__( 'Name', 'events-optimizer' ),
        'post_content'   => esc_html__( 'Description', 'events-optimizer' ),
        '_VenueAddress'  => esc_html__( 'Address', 'events-optimizer' ),
        '_VenueZip'      => esc_html__( 'ZIP', 'events-optimizer' ),
        '_VenueCity'     => esc_html__( 'City', 'events-optimizer' ),
        '_VenueProvince' => esc_html__( 'Province', 'events-optimizer' ),
        '_VenueState'    => esc_html__( 'State', 'events-optimizer' ),
        '_VenueCountry'  => esc_html__( 'Country', 'events-optimizer' ),
        '_VenuePhone'    => esc_html__( 'Phone', 'events-optimizer' ),
        '_VenueURL'      => esc_html__( 'Website', 'events-optimizer' ),
    );
    // Get the next index of the venue if function is called by "Show more" button
    $next_result = ( filter_input( INPUT_POST, 'next_result', FILTER_SANITIZE_NUMBER_INT ) ?: 0 );
    // Set index to the next upcoming venue
    $k = $next_result;
    $venue = tribe_get_venues();
    $c = count( $venue );
    // Maximum amount of venues
    $i = 0;
    // Index to count for maximum results per page
    // Check if not all venues have been shown yet and maximum results has not been reached
    while ( $k < $c and $i < $max_results_per_page ) {
        // Hide venues that have no errors
        $found_error = false;
        $venue_details = get_post_meta( $venue[$k]->ID ) + array(
            'ID'           => $venue[$k]->ID,
            'post_title'   => sanitize_text_field( $venue[$k]->post_title ),
            'post_content' => sanitize_text_field( $venue[$k]->post_content ),
        );
        // If venue isn't part of the ignored or added objects list show it
        if ( array_search( $venue[$k]->ID, array_column( $ignored_and_added_objects, 'post_id' ) ) === false or $show_ignored_objects === 'true' or $show_added_objects === 'true' ) {
            echo '<form action="" method="POST" id="save_venue_wid_form_' . esc_attr( $k ) . '"><br>
				<table class="widefat fixed striped">
				<thead>
				<tr>
				<th>' . esc_html__( 'Field', 'events-optimizer' ) . '</th>
				<th>' . esc_html__( 'Value', 'events-optimizer' ) . '</th>
				<th>' . esc_html__( 'Status', 'events-optimizer' ) . '</th>
				</tr>
				</thead>
				<tbody>';
            foreach ( $fields_arr as $field => $fieldname ) {
                // Get value of Events Calendar field
                // It should always an string but somehow sometimes it is an array
                if ( isset( $venue_details[$field] ) ) {
                    if ( is_array( $venue_details[$field] ) ) {
                        // Convert the array to a string
                        $value = implode( ', ', $venue_details[$field] );
                    } else {
                        $value = sanitize_text_field( (string) $venue_details[$field] );
                    }
                } else {
                    $value = '';
                }
                switch ( $field ) {
                    case 'ID':
                        if ( isset( $venue[$k] ) && isset( $venue[$k]->ID ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><input type="hidden" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '"/><td><a href="' . esc_url( '/wp-admin/post.php?post=' . esc_attr( $venue[$k]->ID ) . '&action=edit' ) . '" target="_blank" rel="noopener noreferrer">' . esc_attr( $value ) . ' <img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png' ) . '" class="external-link-icon" alt="External Link"/></a></td><td>--</td></tr>';
                        }
                        break;
                    case 'post_title':
                        if ( isset( $venue[$k] ) && isset( $venue[$k]->post_title ) && isset( $venue[$k]->_VenueCity ) ) {
                            if ( empty( $value ) ) {
                                echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( sanitize_text_field( $venue[$k]->post_title ) . ' ' . sanitize_text_field( $venue[$k]->_VenueCity ) ) . '" target="_blank" rel="noopener noreferrer"><img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                                $found_error = true;
                            } else {
                                echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( sanitize_text_field( $venue[$k]->post_title ) . ' ' . sanitize_text_field( $venue[$k]->_VenueCity ) ) . '" target="_blank" rel="noopener noreferrer"><img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td></td></tr>';
                            }
                            break;
                        }
                    case 'post_content':
                        if ( empty( $value ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><textarea name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" rows="6" cols="30"></textarea></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                            if ( $ignored_field_venue_description === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><textarea name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" rows="6" cols="30">' . esc_textarea( $value ) . '</textarea></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    case '_VenueZip':
                        if ( empty( $value ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                            if ( $ignored_field_venue_zip === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    // Only for Non-USA countries: set the province (= Bundesland)
                    case '_VenueProvince':
                        if ( isset( $venue_details['_VenueCountry'][0] ) and sanitize_text_field( $venue_details['_VenueCountry'][0] ) != 'United States' ) {
                            if ( empty( $value ) ) {
                                echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                                if ( $ignored_field_venue_province === 'false' ) {
                                    $found_error = true;
                                }
                            } else {
                                echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                            }
                        }
                        break;
                    // Only for USA - set abbreviation of state, e.g. AZ for Arizona)
                    case '_VenueState':
                        if ( isset( $venue_details['_VenueCountry'][0] ) and $venue_details['_VenueCountry'][0] === 'United States' ) {
                            if ( empty( $value ) ) {
                                echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                                if ( $ignored_field_venue_state === 'false' ) {
                                    $found_error = true;
                                }
                            } else {
                                echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                            }
                        }
                        break;
                    case '_VenueCountry':
                        if ( empty( $value ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                            if ( $ignored_field_venue_country === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            // Custom countries available?
                            if ( !empty( $countries_arr ) ) {
                                $value = sanitize_text_field( $value );
                                if ( !in_array( $value, $countries_arr ) ) {
                                    echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . esc_html__( 'Country not in custom country list', 'events-optimizer' ) . '</span></td></tr>';
                                    if ( $ignored_field_venue_country === 'false' ) {
                                        $found_error = true;
                                    }
                                } else {
                                    echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                                }
                            } else {
                                echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                            }
                        }
                        break;
                    // Check if country is in custom country list
                    case '_VenueCountry':
                        if ( empty( $value ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                            if ( $ignored_field_venue_country === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            // Custom countries available?
                            if ( !empty( $countries_arr ) ) {
                                if ( !in_array( $value, $countries_arr ) ) {
                                    echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . esc_html__( 'Country not in custom country list', 'events-optimizer' ) . '</span></td></tr>';
                                    if ( $ignored_field_venue_country === 'false' ) {
                                        $found_error = true;
                                    }
                                } else {
                                    echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                                }
                            } else {
                                echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                            }
                        }
                        break;
                    case '_VenuePhone':
                        $valid = phone_number_valid( $value );
                        if ( $valid != '' ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="' . esc_url( 'https://www.google.com/search?q=' . urlencode( sanitize_text_field( $venue[$k]->post_title ) . ' ' . sanitize_text_field( $venue[$k]->_VenueCity ) ) . ' phone' ) . '" target="_blank" rel="noopener noreferrer"> <img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . esc_html( $valid ) . '</span></td></tr>';
                            if ( $ignored_field_venue_phone === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="' . esc_url( 'https://www.google.de/search?q=' . urlencode( sanitize_text_field( $venue[$k]->post_title ) . ' phone' ) ) . '" target="_blank" rel="noopener noreferrer"> <img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    case '_VenueURL':
                        $valid = url_valid( $value );
                        if ( $valid['status'] != 'OK' ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="' . esc_url( 'https://www.google.com/search?q=' . urlencode( sanitize_text_field( $venue[$k]->post_title ) . ' ' . sanitize_text_field( $venue[$k]->_VenueCity ) ) ) . '" target="_blank" rel="noopener noreferrer"><img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . esc_html( $valid['error_message'] ?? '' ) . '</span></td></tr>';
                            if ( $ignored_field_venue_website === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="' . esc_url( $value ) . '" target="_blank" rel="noopener noreferrer"><img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png' ) . '" class="external-link-icon" alt="External Link"/></a></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    default:
                        if ( empty( $value ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                            if ( $field === 'post_title' ) {
                                $found_error = true;
                            }
                            if ( $field === '_VenueAddress' and $ignored_field_venue_address === 'false' ) {
                                $found_error = true;
                            }
                            if ( $field === '_VenueCity' and $ignored_field_venue_city === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                }
                // end switch
            }
            // end foreach
            echo '</tbody>
					</table>
					<p><input type="button" name="action" class="button button-primary" id="save_venue_wid_button_' . esc_attr( $k ) . '" value="' . esc_html__( 'Save changes', 'events-optimizer' ) . '" onclick="save_form(' . "'" . esc_attr( $k ) . "'" . ');" />
					&nbsp;&nbsp;&nbsp;<input type="button" name="action" class="button delete" id="ignore_object_button_' . esc_attr( $k ) . '" value="' . esc_html__( 'Ignore', 'events-optimizer' ) . '" onclick="ignore_object(' . "'" . esc_attr( $k ) . "'" . ');"';
            disabled( !teo_fs()->can_use_premium_code__premium_only() );
            echo '/>
					</p><div id="status_save_venue_wid_' . esc_attr( $k ) . '">' . esc_html__( 'SAVING', 'events-optimizer' ) . '</div></p><br>
					</form>';
        }
        // end if array_search(
        $nonce = wp_create_nonce( 'events-optimizer-nonce' );
        ?>	
			
			<script  type='text/javascript'>
		
				function save_form(form_id){
					var datastring = jQuery("#save_venue_wid_form_"+form_id).serialize();
					jQuery("#save_venue_wid_button_"+form_id).attr("disabled", true); // Disable button while saving
					jQuery.ajax({
						type: "POST",
						url: "<?php 
        echo admin_url( 'admin-ajax.php' );
        ?>",
						data: { 
							action: 'save_ven',
							_ajax_nonce: '<?php 
        echo esc_attr( $nonce );
        ?>',
							data: datastring,
						},
						beforeSend: function() {
							jQuery("#status_save_venue_wid_"+form_id).fadeIn('fast');  // Fade in status when button is clicked
						},
						success: function(html) {
							jQuery("#status_save_venue_wid_"+form_id).fadeOut('slow');	// Fade out status after data is saved
							jQuery("#save_venue_wid_button_"+form_id).attr("disabled", false); // Enable button after saving
							jQuery("#save_venue_wid_form_"+form_id).slideUp(); // Disable venue form after saving							
						}
					}); //close jQuery.ajax
					return false;	
				}
				function ignore_object(form_id){	
					var datastring = jQuery("#save_venue_wid_form_"+form_id).serialize() + '&type=tribe_venue&status=ignored';
					jQuery("#ignore_object_button_"+form_id).attr("disabled", true); // Disable button while ignoring
					jQuery.ajax({
						type: "POST",
						url: "<?php 
        echo admin_url( 'admin-ajax.php' );
        ?>",
						data: { 
							action: 'add_obj',
							_ajax_nonce: '<?php 
        echo esc_attr( $nonce );
        ?>',
							data: datastring,
						},
						beforeSend: function() {
						},
						success: function(html) {
							jQuery("#status_save_venue_wid_"+form_id).fadeOut('slow');	// Fade out status after data is ignored
							jQuery("#ignore_object_button_"+form_id).attr("disabled", false); // Enable button after ignoring
							jQuery("#save_venue_wid_form_"+form_id).slideUp(); // Slide venue form away after ignoring							
						}
					}); //close jQuery.ajax
					return false;	
				}
			</script>	

			<style type='text/css'>
				#status_save_venue_wid_<?php 
        esc_attr_e( $k );
        ?> { 
					display: none;
					clear: both;
					background: url(<?php 
        echo esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' );
        ?>) left top no-repeat; 
					text-align: left;
					vertical-align: center;	
					padding-left: 25px;
				}
				<?php 
        if ( $found_error === false ) {
            echo "#save_venue_wid_form_" . esc_attr( $k ) . ", #save_venue_wid_button_" . esc_attr( $k ) . " { display: none; }";
        }
        ?>
			</style>

			<?php 
        // if invalid data is found increase counter for maximum results per page
        if ( $found_error === true ) {
            $i++;
        }
        $k++;
    }
    // end while
    if ( $k < $c ) {
        echo '<div id="additional_results_venues_wid_' . esc_attr( $k ) . '"><p><strong>' . esc_html__( 'Additional results available.', 'events-optimizer' ) . '</strong></p></div>';
        echo '<p><input type="submit" name="action" class="button button-primary" id="show_more_venues_wid_button_' . esc_attr( $k ) . '" value="' . esc_html__( 'Show more', 'events-optimizer' ) . '" onclick="show_more(' . "'" . esc_attr( $k ) . "'" . ');" /></p>';
        echo '<div id="show_more_venues_wid"></div>';
        echo '<div id="status_show_more_venues_wid_' . esc_attr( $k ) . '">' . esc_html__( 'ANALYZING', 'events-optimizer' ) . '</div>';
    }
    if ( $next_result + $max_results_per_page >= $c ) {
        echo '<span class="field-status-ok">' . esc_html__( 'No (more) results found!', 'events-optimizer' ) . '</span>';
    }
    $nonce = wp_create_nonce( 'events-optimizer-nonce' );
    ?>	
			
		<script  type='text/javascript'>
		
			function show_more(form_id){
				jQuery("#show_more_venues_wid_button_"+form_id).fadeOut('slow'); // Hide button after  		
				jQuery("#show_more_venues_wid_button_"+form_id).attr("disabled", true); // Disable button while retrieving more results
				jQuery.ajax({
					type: "POST",
					url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
					data: { 
						action: 'venues_wid',
						next_result: form_id,
						_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>' 
					},
					beforeSend: function() {
						jQuery("#status_show_more_venues_wid_"+form_id).fadeIn('fast');  // Fade in status when button is clicked
						jQuery("#additional_results_venues_wid_"+form_id).fadeOut('slow');	// Fade out text "Additional results ..." after data is saved
					},
					success: function(html) {
						jQuery("#show_more_venues_wid_button_"+form_id).attr("disabled", false); // Enable button after
						jQuery("#status_show_more_venues_wid_"+form_id).fadeOut('slow');	// Fade out status after data is saved
						jQuery("#show_more_venues_wid").append(html); // Add the additional results to page
					}
				}); //close jQuery.ajax
				return false;	
			}	
		</script>

		<style type='text/css'>
			#status_show_more_venues_wid_<?php 
    esc_attr_e( $k );
    ?> { 
				display: none;
				clear: both;
				background: url(<?php 
    echo esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' );
    ?>) left top no-repeat; 
				text-align: left;
				vertical-align: center;	
				padding-left: 25px;
			}
		</style>

		<?php 
    wp_die();
}

//----------------------------------------------------------------------------------------
// Venues with inactive business operation
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_venues_with_inactive_bo', 'venues_with_inactive_business_operation' );
function venues_with_inactive_business_operation() {
}

//----------------------------------------------------------------------------------------
// Create Venue List Table
//----------------------------------------------------------------------------------------
class Venue_List_Table extends WP_List_Table {
    public $venue_arr = array();

    // Define the columns for the table
    function get_columns() {
        $columns = array(
            'cb'              => '<input type="checkbox" />',
            'name'            => esc_html__( 'Name', 'events-optimizer' ),
            'address'         => esc_html__( 'Address', 'events-optimizer' ),
            'business_status' => esc_html__( 'Business Status', 'events-optimizer' ),
        );
        return $columns;
    }

    // Prepare the items for the table
    function prepare_items() {
        $columns = $this->get_columns();
        $hidden = $this->get_hidden_columns();
        $sortable = $this->get_sortable_columns();
        $this->_column_headers = array($columns, $hidden, $sortable);
        // Get the current page number
        $current_page = $this->get_pagenum();
        // Maximum number of items to display per page
        $max_results_per_page = intval( get_option( 'teo_max_results_per_page', 5 ) );
        // Determine the offset for the query based on the current page number
        $offset = ($current_page - 1) * $max_results_per_page;
        // Get the total number of items
        $total_items = count( $this->venue_arr );
        // Generieren Sie die Paginierungslinks
        $pagination_args = array(
            'total_items' => $total_items,
            'per_page'    => $max_results_per_page,
            'total_pages' => ceil( $total_items / $max_results_per_page ),
        );
        $this->set_pagination_args( $pagination_args );
        // Slice data to retrieve only items for the current page
        $this->venue_arr = array_slice( $this->venue_arr, $offset, $max_results_per_page );
        // Get the business status for each venue when preparing the items for the table
        foreach ( $this->venue_arr as &$item ) {
            $result = google_places_text_search( implode( ' ', array(
                $item['name'],
                $item['address'],
                $item['city'],
                $item['country']
            ) ) );
            switch ( $result['status'] ) {
                case 'OK':
                    $item['business_status'] = sanitize_text_field( $result['results'][0]['business_status'] );
                    break;
                case 'ZERO_RESULTS':
                    $item['business_status'] = wp_kses_post( $result['status'] );
                    break;
                default:
                    $item['business_status'] = '<strong>' . esc_html__( 'Status: ', 'events-optimizer' ) . '</strong>' . wp_kses_post( $result['status'] ?? '' ) . '<br><strong>' . esc_html__( 'Error message: ', 'events-optimizer' ) . '</strong>' . wp_kses_post( $result['error_message'] ?? '' );
            }
        }
        $this->items = $this->venue_arr;
    }

    // Hide columns
    function get_hidden_columns() {
        // Return an array of hidden columns (none at the moment)
        return array();
    }

    // Render the checkbox column
    function column_cb( $item ) {
        // if business status is not operational, pre-check the checkbox
        if ( $item['business_status'] !== 'OPERATIONAL' ) {
            return sprintf( '<input type="checkbox" name="ids[]" value="%s" checked />', $item['ID'] );
        } else {
            return sprintf( '<input type="checkbox" name="ids[]" value="%s" />', $item['ID'] );
        }
    }

    // Render the other columns
    function column_default( $item, $column_name ) {
        switch ( $column_name ) {
            case 'cb':
                return $this->column_cb( $item );
            case 'address':
                // show address, city and country in one column
                return implode( ', ', array_filter( [$item['address'], $item['city'], $item['country']] ) );
            case 'business_status':
                if ( $item['business_status'] === 'OPERATIONAL' ) {
                    return '<span class="field-status-ok">' . esc_html( $item['business_status'] ) . '</span>';
                } else {
                    return '<span class="field-status-error">' . wp_kses_post( $item['business_status'] ) . '</span>';
                }
            case 'name':
                return sprintf( '<a href="%s">%s</a>', get_edit_post_link( $item['ID'] ), sanitize_text_field( $item['name'] ) );
            default:
                return print_r( $item, true );
        }
    }

    // Define the bulk actions
    function get_bulk_actions() {
        $actions = array(
            'trash'   => esc_html__( 'Move to Trash', 'events-optimizer' ),
            'draft'   => esc_html__( 'Change Status to Draft', 'events-optimizer' ),
            'pending' => esc_html__( 'Change Status to Pending Review', 'events-optimizer' ),
        );
        return $actions;
    }

    // Process the bulk actions
    function process_bulk_actions() {
        // Check if the user has the necessary permissions
        if ( !(current_user_can( 'administrator' ) or current_user_can( sanitize_text_field( get_option( 'teo_allowed_role' ) ) )) ) {
            wp_die( esc_html__( 'You do not have sufficient permissions to access this page.', 'events-optimizer' ) );
        }
        $venue_ids = ( isset( $_REQUEST['ids'] ) ? array_filter( $_REQUEST['ids'], 'is_numeric' ) : array() );
        $bulk_action = ( isset( $_REQUEST['bulk_action'] ) ? sanitize_text_field( $_REQUEST['bulk_action'] ) : null );
        switch ( $bulk_action ) {
            case 'trash':
                // Logic to move items to trash
                if ( is_array( $venue_ids ) ) {
                    foreach ( $venue_ids as $venue_id ) {
                        // Actual logic to move the venue to trash
                        wp_trash_post( $venue_id );
                    }
                }
                break;
            case 'draft':
                // Logic to change the status of items
                if ( is_array( $venue_ids ) ) {
                    foreach ( $venue_ids as $venue_id ) {
                        // Actual logic to change the status of the venue
                        wp_update_post( array(
                            'ID'          => $venue_id,
                            'post_status' => 'draft',
                        ) );
                    }
                }
                break;
            case 'pending':
                // Logic to change the status of items to pending review
                if ( is_array( $venue_ids ) ) {
                    foreach ( $venue_ids as $venue_id ) {
                        // Actual logic to change the status of the venue
                        wp_update_post( array(
                            'ID'          => $venue_id,
                            'post_status' => 'pending',
                        ) );
                    }
                }
                break;
            default:
                // no action
                break;
        }
    }

    // Display the table
    function display() {
        // Check if the user has the necessary permissions
        if ( !(current_user_can( 'administrator' ) or current_user_can( sanitize_text_field( get_option( 'teo_allowed_role' ) ) )) ) {
            wp_die( esc_html__( 'You do not have sufficient permissions to access this page.', 'events-optimizer' ) );
        }
        // Start of the form
        echo '<form action="" method="POST" id="venues-filter" >';
        // Display the top table navigation (if any)
        $this->display_tablenav( 'top' );
        // Open the table
        echo '<table class="wp-list-table ' . implode( ' ', $this->get_table_classes() ) . '">';
        // Display the table headers
        echo '<thead>';
        $this->print_column_headers();
        echo '</thead>';
        // Open the table body
        echo '<tbody id="the-list">';
        // Display the table rows
        $this->display_rows_or_placeholder();
        // Close the table body
        echo '</tbody>';
        // Display the table footers
        echo '<tfoot>';
        $this->print_column_headers( false );
        echo '</tfoot>';
        // Close the table
        echo '</table>';
        // Display the bottom table navigation (if any)
        $this->display_tablenav( 'bottom' );
        // Close the form
        echo '</form>';
    }

    function get_table_classes() {
        // Change the width of the columns (remove "striped")
        return array('widefat', 'striped', $this->_args['plural']);
    }

}

//----------------------------------------------------------------------------------------
// Save venue data (for invalid data and Google Places optimization)
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_save_ven', 'save_venue' );
function save_venue() {
    // Check the nonce
    if ( !check_ajax_referer( 'events-optimizer-nonce', '_wpnonce', false ) ) {
        wp_die( 'Invalid nonce' );
    }
    parse_str( $_POST['data'], $venue );
    // Title and description can't be updated directly with the TEC API (empty fields gets ignored - bug?) and therefore it needs to be updated seperatly
    $venue_post_arr = array(
        'ID'           => sanitize_text_field( $venue['ID'] ),
        'post_title'   => sanitize_text_field( $venue['post_title'] ),
        'post_content' => wp_kses_post( $venue['post_content'] ),
    );
    wp_update_post( $venue_post_arr );
    // Check if state and province are set
    if ( !isset( $venue['_VenueState'] ) ) {
        $venue['_VenueState'] = '';
    }
    if ( !isset( $venue['_VenueProvince'] ) ) {
        $venue['_VenueProvince'] = '';
    }
    // Update venues with values
    $venue_arr = array(
        'Address'       => sanitize_text_field( $venue['_VenueAddress'] ),
        'GeoAddress'    => sanitize_text_field( $venue['_VenueAddress'] ) . ' ' . sanitize_text_field( $venue['_VenueZip'] ) . ' ' . sanitize_text_field( $venue['_VenueCity'] ) . ' ' . sanitize_text_field( $venue['_VenueCountry'] ),
        'Zip'           => sanitize_text_field( $venue['_VenueZip'] ),
        'City'          => sanitize_text_field( $venue['_VenueCity'] ),
        'State'         => sanitize_text_field( $venue['_VenueState'] ),
        'Province'      => sanitize_text_field( $venue['_VenueProvince'] ),
        'StateProvince' => sanitize_text_field( $venue['_VenueProvince'] ),
        'Country'       => sanitize_text_field( $venue['_VenueCountry'] ),
        'Phone'         => sanitize_text_field( $venue['_VenuePhone'] ),
        'URL'           => esc_url( $venue['_VenueURL'] ),
    );
    tribe_update_venue( $venue['ID'], $venue_arr );
    wp_die();
}

//----------------------------------------------------------------------------------------
// Duplicate Venues
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_duplicate_ven', 'get_duplicate_venues' );
function get_duplicate_venues() {
}

//----------------------------------------------------------------------------------------
// Merge Venues
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_merge_ven', 'merge_venues' );
function merge_venues() {
}

//----------------------------------------------------------------------------------------
// Venues: Optimization with Google Places
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_venues_optimization_wgp', 'venues_optimization_with_google_places' );
function venues_optimization_with_google_places() {
}

//----------------------------------------------------------------------------------------
// Generate description for single venue
//----------------------------------------------------------------------------------------
function generate_venue_description() {
    // Activate only for Pro version
    if ( teo_fs()->can_use_premium_code__premium_only() and get_option( 'teo_openai_hide_generate_description_buttons' ) === 'false' ) {
        $nonce = wp_create_nonce( 'events-optimizer-nonce' );
        wp_enqueue_style( 'style.css', plugin_dir_url( __FILE__ ) . 'assets/css/style.css' );
        // Add button to existing meta box inside organizer form
        ?>	
		
			<input type="button" name="action" id="generate_venue_description_button" class="button-primary" value="<?php 
        \esc_attr_e( 'Generate description', 'events-optimizer' );
        ?>" />
			<span class="field-tip">
				&nbsp;<span class="question-mark-centered"><img src ="<?php 
        echo esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/question-mark.png' );
        ?>"></span>
				<span class="tip-content"><?php 
        esc_html_e( 'Venue name and city are used to generate the description. Please enter at least the name!', 'events-optimizer' );
        ?></span>&nbsp;&nbsp;
				<span id="status_generate_event_description_single_venue"><?php 
        esc_html_e( 'GENERATING', 'events-optimizer' );
        ?></span>
			</span>

			<script type='text/javascript'>

				jQuery(document).ready(function() {

					jQuery("#generate_venue_description_button").on('click', function()  {
						
						jQuery("#generate_venue_description_button").attr("disabled", true); // Disable button while retrieving more results

						// Get venue website URL
						url = document.getElementById("EventWebsite").value;
						
						// Get existing content from TinyMCE editor or text editor
						if (typeof tinyMCE !== "undefined" && tinyMCE.activeEditor != null && !tinyMCE.activeEditor.isHidden()) {	
							existing_content = tinymce.activeEditor.getContent();
						} else if ( jQuery('#content').length > 0 ) { 
							existing_content = escapeHTML( jQuery('#content').val() );
						}
						
						searchstring =	escapeHTML(
										document.getElementById('title').value	+ " " +
										document.getElementById('venueAddress').value + " " +
										document.getElementById('venueCity').value + " " +
										document.getElementById('EventZip').value + " " +
										document.getElementById('EventCountry').value
						);

						// Check if searchstring is empty
						if ( searchstring.trim() === "" || document.getElementById("title").value === "" ) { 
							alert( 'Venue name, address, ZIP, city and country used to generate the description. Please enter at least the name!', 'events_optimizer' );
							jQuery("#generate_venue_description_button").attr("disabled", false);// Enable button after	
						} else {							
						
							jQuery.ajax({
								type: "POST",
								url: "<?php 
        echo admin_url( 'admin-ajax.php' );
        ?>",
								data: { 		
									action: 'generate_venue_description_with_oai', // call function "generate_venue_description_with_oai"					
									_ajax_nonce: '<?php 
        echo esc_attr( $nonce );
        ?>',
									searchstring: searchstring,
									existing_content: existing_content,
									url: url
								},
								beforeSend: function() {
									jQuery("#status_generate_event_description_single_venue").fadeIn('fast');  // Fade in status when button is clicked								
								},
								success : function( data ) {
									info = JSON.parse( data );
									
									// Error handling
									if ( info.error_message ) {
										alert("Status: "+info.status+"\nError message: "+info.error_message);
									} else {
										// Check if info.description is defined and not empty
										if (info.description && info.description.trim() !== "") {
											// Add generated description to content field (either TinymCE or text editor)
											if (typeof tinyMCE !== "undefined" && tinyMCE.activeEditor != null && !tinyMCE.activeEditor.isHidden()) {
												tinyMCE.activeEditor.setContent(info.description);
											} else if (jQuery("#content").length > 0) {
												jQuery("#content").val(info.description);
											} else {
												alert("Unable to set content: No suitable editor or text area found");
											}
										} else {
											alert("No description retrieved. Please check the search string and try again.");
										}
									}
									
									jQuery("#generate_venue_description_button").attr("disabled", false); // Enable button after
									jQuery("#status_generate_event_description_single_venue").fadeOut('slow');	// Fade out status after data is saved
								}
							})
						}
						return false;					
					});											
				});
				
			</script>

			<style type='text/css'>
				#status_generate_event_description_single_venue { 
					display: none;
					clear: both;
					background: url(<?php 
        echo esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' );
        ?>) left top no-repeat; 
					text-align: left;
					vertical-align: center;	
					padding-left: 25px;
				}
			</style>
		
			<?php 
    }
}

//----------------------------------------------------------------------------------------
// Generate venue description and hand it over to Java Script function above
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_generate_venue_description_with_oai', 'generate_venue_description_with_open_ai' );
function generate_venue_description_with_open_ai() {
}

//----------------------------------------------------------------------------------------
// Fetch data from Google Places for single venue
//----------------------------------------------------------------------------------------
add_action( 'tribe_events_venue_before_metabox', 'fetch_venue_data' );
function fetch_venue_data() {
}

//----------------------------------------------------------------------------------------
// Get venue data from Google Places and hand it over to Java Script function above
// https://www.codexworld.com/post-get-json-data-from-php-script-jquery-ajax/
// https://www.mediaevent.de/javascript/json.html
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_fetch_venue_data_fgp', 'fetch_venue_data_from_google_places' );
function fetch_venue_data_from_google_places() {
}

//----------------------------------------------------------------------------------------
// Organizers with invalid data
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_organizers_wid', 'organizers_with_invalid_data' );
function organizers_with_invalid_data() {
    // Check the nonce
    if ( !check_ajax_referer( 'events-optimizer-nonce', '_wpnonce', false ) ) {
        wp_die( 'Invalid nonce' );
    }
    // Maximum number of results to display per page
    $max_results_per_page = intval( get_option( 'teo_max_results_per_page', 5 ) );
    // Objects to be ignored while searching for invalid data
    $ignored_and_added_objects = maybe_unserialize( get_option( 'teo_ignored_and_added_objects' ) );
    if ( empty( $ignored_and_added_objects ) ) {
        $ignored_and_added_objects = array();
    }
    // Whether ignored objects should be shown or not
    $show_ignored_objects = get_option( 'teo_show_ignored_objects' );
    // Whether added objects should be shown or not
    $show_added_objects = get_option( 'teo_show_added_objects' );
    // Fields to be ignored while searching for invalid data
    $ignored_field_organizer_description = get_option( 'teo_ignored_field_organizer_description' );
    $ignored_field_organizer_phone = get_option( 'teo_ignored_field_organizer_phone' );
    $ignored_field_organizer_website = get_option( 'teo_ignored_field_organizer_website' );
    $ignored_field_organizer_mail = get_option( 'teo_ignored_field_organizer_mail' );
    $fields_arr = array(
        'ID'                => esc_html__( 'ID', 'events-optimizer' ),
        'post_title'        => esc_html__( 'Name', 'events-optimizer' ),
        'post_content'      => esc_html__( 'Description', 'events-optimizer' ),
        '_OrganizerPhone'   => esc_html__( 'Phone', 'events-optimizer' ),
        '_OrganizerWebsite' => esc_html__( 'Website', 'events-optimizer' ),
        '_OrganizerEmail'   => esc_html__( 'Mail', 'events-optimizer' ),
    );
    // Get the next index of the venue if function is called by "Show more" button
    $next_result = ( filter_input( INPUT_POST, 'next_result', FILTER_SANITIZE_NUMBER_INT ) ?: 0 );
    // Set index to the next upcoming venue
    $k = $next_result;
    $organizer = tribe_get_organizers();
    $c = count( $organizer );
    // Maximum amount of organizers
    $i = 0;
    // Index to count for maximum results per page
    // Check if not all organizers have been shown yet and maximum results has not been reached
    while ( $k < $c and $i < $max_results_per_page ) {
        // Hide organizers that have no errors
        $found_error = false;
        $organizer_details = get_post_meta( $organizer[$k]->ID ) + array(
            'ID'           => array($organizer[$k]->ID),
            'post_title'   => array($organizer[$k]->post_title),
            'post_content' => array($organizer[$k]->post_content),
        );
        // If organizer isn't part of the ignored or added objects list show it
        if ( array_search( $organizer[$k]->ID, array_column( $ignored_and_added_objects, 'post_id' ) ) === false or $show_ignored_objects === 'true' or $show_added_objects === 'true' ) {
            echo '<form action="" method="POST" id="save_organizer_wid_form_' . esc_attr( $k ) . '">
				<br>		
				<table class="widefat fixed striped">
				<thead>
				<tr>
				<th>' . esc_html__( 'Field', 'events-optimizer' ) . '</th>
				<th>' . esc_html__( 'Value', 'events-optimizer' ) . '</th>
				<th>' . esc_html__( 'Status', 'events-optimizer' ) . '</th>
				</tr>
				</thead>
				<tbody>';
            foreach ( $fields_arr as $field => $fieldname ) {
                if ( isset( $organizer_details[$field][0] ) ) {
                    $value = $organizer_details[$field][0];
                } else {
                    $value = '';
                }
                echo '<input type="hidden" name="index" id="index" value="' . esc_attr( $k ) . '"/>';
                switch ( $field ) {
                    case 'ID':
                        echo '<tr><td>' . esc_attr( $fieldname ) . '</td><input type="hidden" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '"/><td><a href="/wp-admin/post.php?post=' . esc_attr( $organizer[$k]->ID ) . '&action=edit" target="_blank" rel="noopener noreferrer">' . esc_attr( $value ) . ' <img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png' ) . '" class="external-link-icon" alt="External Link"/></a></td><td>--</td></tr>';
                        break;
                    case 'post_title':
                        if ( empty( $value ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $organizer[$k]->post_title ) . '" target="_blank" rel="noopener noreferrer"><img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $organizer[$k]->post_title ) . '" target="_blank" rel="noopener noreferrer"><img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    case 'post_content':
                        if ( empty( $value ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><textarea name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" rows="6" cols="30"></textarea></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                            if ( $ignored_field_organizer_description === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><textarea name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" rows="6" cols="30">' . esc_textarea( $value ) . '</textarea></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    case '_OrganizerPhone':
                        $valid = phone_number_valid( $value );
                        if ( $valid != '' ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $organizer[$k]->post_title . ' phone' ) . '" target="_blank" rel="noopener noreferrer"><img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . esc_attr( $valid ?? '' ) . '</td></tr>';
                            if ( $ignored_field_organizer_phone === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $organizer[$k]->post_title ) . ' phone" target="_blank" rel="noopener noreferrer"><img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    case '_OrganizerWebsite':
                        $valid = url_valid( $value );
                        if ( $valid['status'] != 'OK' ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $organizer[$k]->post_title ) . '" target="_blank" rel="noopener noreferrer"><img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . esc_attr( $valid['error_message'] ?? '' ) . '</td></tr>';
                            if ( $ignored_field_organizer_website === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="' . urlencode( $value ) . '" target="_blank" rel="noopener noreferrer"> <img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png' ) . '" class="external-link-icon" alt="External Link"/></a></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    // Check if mail is valid
                    case '_OrganizerEmail':
                        $valid = mail_valid( $value );
                        if ( $valid != '' ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $organizer[$k]->post_title . ' mail' ) . '" target="_blank" rel="noopener noreferrer"><img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . esc_attr( $valid ?? '' ) . '</td></tr>';
                            if ( $ignored_field_organizer_mail === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $organizer[$k]->post_title . ' mail' ) . '" target="_blank" rel="noopener noreferrer"><img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    default:
                        if ( empty( $value ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $organizer[$k]->post_title ) . '" target="_blank" rel="noopener noreferrer"><img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                }
                // end switch
            }
            // end foreach
            echo '</tbody>
					</table>	
					<p><input type="button" name="action" class="button button-primary" id="save_organizer_wid_button_' . esc_attr( $k ) . '" value="' . esc_html__( 'Save changes', 'events-optimizer' ) . '" onclick="save_form(' . "'" . esc_attr( $k ) . "'" . ');" />
					&nbsp;&nbsp;&nbsp;<input type="button" name="action" class="button delete" id="ignore_object_button_' . esc_attr( $k ) . '" value="' . esc_html__( 'Ignore', 'events-optimizer' ) . '" onclick="ignore_object(' . "'" . esc_attr( $k ) . "'" . ');"';
            disabled( !teo_fs()->can_use_premium_code__premium_only() );
            echo '/>
					</p><div id="status_save_organizer_wid_' . esc_attr( $k ) . '">' . esc_html__( 'SAVING', 'events-optimizer' ) . '</div></p><br>
					</form>';
        }
        // end if array_search(
        $nonce = wp_create_nonce( 'events-optimizer-nonce' );
        ?>	

			<script  type='text/javascript'>
		
				function save_form(form_id){
					var datastring = jQuery("#save_organizer_wid_form_"+form_id).serialize(); 
					jQuery("#save_organizer_wid_button_"+form_id).attr("disabled", true); // Disable button while saving
					jQuery.ajax({
						type: "POST",
						url: "<?php 
        echo admin_url( 'admin-ajax.php' );
        ?>",
						data: { 
							action: 'save_org',
							_ajax_nonce: '<?php 
        echo esc_attr( $nonce );
        ?>',
							data: datastring,
							},
						beforeSend: function() {
							jQuery("#status_save_organizer_wid_"+form_id).fadeIn('fast');  // Fade in status when button is clicked
							},
						success: function(html) {
							jQuery("#status_save_organizer_wid_"+form_id).fadeOut('slow');	// Fade out status after data is saved
							jQuery("#save_organizer_wid_button_"+form_id).attr("disabled", false); // Enable button after saving
							jQuery("#save_organizer_wid_form_"+form_id).slideUp(); // Slide organizer form away after saving				
						}
					}); //close jQuery.ajax
					return false;	
				}
				function ignore_object(form_id){
					var datastring = jQuery("#save_organizer_wid_form_"+form_id).serialize() + '&type=tribe_organizer&status=ignored';
					jQuery("#ignore_object_button_"+form_id).attr("disabled", true); // Disable button while ignoring
					jQuery.ajax({
						type: "POST",
						url: "<?php 
        echo admin_url( 'admin-ajax.php' );
        ?>",
						data: { 
							action: 'add_obj',
							_ajax_nonce: '<?php 
        echo esc_attr( $nonce );
        ?>',
							data: datastring,
						},
						beforeSend: function() {
						},
						success: function(html) {
							jQuery("#status_save_organizer_wid_"+form_id).fadeOut('slow');	// Fade out status after data is ignored
							jQuery("#ignore_object_button_"+form_id).attr("disabled", false); // Enable button after ignoring
							jQuery("#save_organizer_wid_form_"+form_id).slideUp(); // Slide organizer form away after ignoring							
						}
					}); //close jQuery.ajax
					return false;	
				}
			</script>	
			
			<style type='text/css'>
				#status_save_organizer_wid_<?php 
        esc_attr_e( $k );
        ?> { 
					display: none;
					clear: both;
					background: url(<?php 
        echo esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' );
        ?>) left top no-repeat; 
					text-align: left;
					vertical-align: center;	
					padding-left: 25px;
				}
				<?php 
        if ( $found_error === false ) {
            echo "#save_organizer_wid_form_" . esc_attr( $k ) . ", #save_organizer_wid_button_" . esc_attr( $k ) . " { display: none; }";
        }
        ?>
			</style>

			<?php 
        // if invalid data is found increase counter for maximum results per page
        if ( $found_error === true ) {
            $i++;
        }
        $k++;
    }
    // end while
    if ( $k < $c ) {
        echo '<div id="additional_results_organizers_wid_' . esc_attr( $k ) . '"><p><strong>' . esc_html__( 'Additional results available.', 'events-optimizer' ) . '</strong></p></div>';
        echo '<p><input type="submit" name="action" class="button button-primary" id="show_more_organizers_wid_button_' . esc_attr( $k ) . '" value="' . esc_html__( 'Show more', 'events-optimizer' ) . '" onclick="show_more(' . "'" . esc_attr( $k ) . "'" . ');" /></p>';
        echo '<div id="show_more_organizers_wid"></div>';
        echo '<div id="status_show_more_organizers_wid_' . esc_attr( $k ) . '">' . esc_html__( 'ANALYZING', 'events-optimizer' ) . '</div>';
    }
    if ( $next_result + $max_results_per_page >= $c ) {
        echo '<span class="field-status-ok">' . esc_html__( 'No (more) results found!', 'events-optimizer' ) . '</span>';
    }
    $nonce = wp_create_nonce( 'events-optimizer-nonce' );
    ?>	
			
		<script  type='text/javascript'>
		
			function show_more(form_id){
				jQuery("#show_more_organizers_wid_button_"+form_id).fadeOut('slow'); // Hide button after  		
				jQuery("#show_more_organizers_wid_button_"+form_id).attr("disabled", true); // Disable button while retrieving more results
				jQuery.ajax({
					type: "POST",
					url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
					data: { 
						action: 'organizers_wid',
						next_result: form_id,
						_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>' 
					},
					beforeSend: function() {
						jQuery("#status_show_more_organizers_wid_"+form_id).fadeIn('fast');  // Fade in status when button is clicked
						jQuery("#additional_results_organizers_wid_"+form_id).fadeOut('slow');	// Fade out text "Additional results ..." after data is saved
					},
					success: function(html) {
						jQuery("#show_more_organizers_button_"+form_id).attr("disabled", false); // Enable button after
						jQuery("#status_show_more_organizers_wid_"+form_id).fadeOut('slow');	// Fade out status after data is saved
						jQuery("#show_more_organizers_wid").append(html); // Add the additional results to page
					}
				}); //close jQuery.ajax
				return false;	
			}	
		</script>

		<style type='text/css'>
			#status_show_more_organizers_wid_<?php 
    esc_attr_e( $k );
    ?> { 
				display: none;
				clear: both;
				background: url(<?php 
    echo esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' );
    ?>) left top no-repeat; 
				text-align: left;
				vertical-align: center;	
				padding-left: 25px;
			}
		</style>

		<?php 
    wp_die();
}

//----------------------------------------------------------------------------------------
// Save organizer data (for invalid data and Optimization with Google Places)
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_save_org', 'save_organizer' );
function save_organizer() {
    // Check the nonce
    if ( !check_ajax_referer( 'events-optimizer-nonce', '_wpnonce', false ) ) {
        wp_die( 'Invalid nonce' );
    }
    parse_str( $_POST['data'], $organizer );
    // Sanitize each value in the array
    foreach ( $organizer as $key => $value ) {
        $organizer[$key] = sanitize_text_field( $value );
    }
    // Title and description are not part of the post_meta therefore they need to be updated seperatly
    $organizer_post_arr = array(
        'ID'           => sanitize_text_field( $organizer['ID'] ),
        'post_title'   => sanitize_text_field( $organizer['post_title'] ),
        'post_content' => wp_kses_post( $organizer['post_content'] ),
    );
    wp_update_post( $organizer_post_arr );
    // Update organizer with values
    $organizer_arr = array(
        'Phone'   => sanitize_text_field( $organizer['_OrganizerPhone'] ),
        'Website' => sanitize_text_field( $organizer['_OrganizerWebsite'] ),
        'Email'   => sanitize_email( $organizer['_OrganizerEmail'] ),
    );
    tribe_update_organizer( $organizer['ID'], $organizer_arr );
    wp_die();
}

//----------------------------------------------------------------------------------------
// Duplicate Organizers
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_duplicate_org', 'duplicate_organizers' );
function duplicate_organizers() {
}

//----------------------------------------------------------------------------------------
// Merge Organizers
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_merge_org', 'merge_organizers' );
function merge_organizers() {
}

//----------------------------------------------------------------------------------------
// Organizers: Optimization with Google Places
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_organizers_optimization_wgp', 'organizers_optimization_with_google_places' );
function organizers_optimization_with_google_places() {
}

//----------------------------------------------------------------------------------------
// Generate description for single organizer
//----------------------------------------------------------------------------------------
function generate_organizer_description() {
    // Activate only for Pro version
    if ( teo_fs()->can_use_premium_code__premium_only() and get_option( 'teo_openai_hide_generate_description_buttons' ) === 'false' ) {
        $nonce = wp_create_nonce( 'events-optimizer-nonce' );
        wp_enqueue_style( 'style.css', plugin_dir_url( __FILE__ ) . 'assets/css/style.css' );
        // Add button to existing meta box inside organizer form
        ?>	
			
			<input type="button" name="action" id="generate_organizer_description_button" class="button-primary" value="<?php 
        esc_attr_e( 'Generate description', 'events-optimizer' );
        ?>" />
			<span class="field-tip">
				&nbsp;<span class="question-mark-centered"><img src ="<?php 
        echo esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/question-mark.png' );
        ?>"></span>
				<span class="tip-content"><?php 
        _e( 'Organizer name is used to generate description. Please enter it (including the city) in the title field!', 'events-optimizer' );
        ?></span>&nbsp;&nbsp;
				<span id="status_generate_event_description_single_organizer"><?php 
        esc_html_e( 'GENERATING', 'events-optimizer' );
        ?></span>
			</span>


			<script type='text/javascript'>

				jQuery(document).ready(function() {

					jQuery("#generate_organizer_description_button").on('click', function()  {

						jQuery("#generate_organizer_description_button").attr("disabled", true); // Disable button while retrieving more results

						// Get organizer website URL
						url = document.getElementById("OrganizerWebsite").value;

						// Get existing content from TinyMCE editor or text editor
						if (typeof tinyMCE !== "undefined" && tinyMCE.activeEditor != null && !tinyMCE.activeEditor.isHidden()) {
							existing_content = tinymce.activeEditor.getContent();
						} else if ( jQuery('#content').length > 0 ) { 
							existing_content = jQuery('#content').val();
						}

						searchstring = escapeHTML( document.getElementById("title").value );

						if ( searchstring.trim() === "" ) { 
							alert ("Organizer name is used to search. Please enter it (including the city) in the title field!");
							jQuery("#generate_organizer_description_button").attr("disabled", false); // Enable button after		
						
						} else {							
						
							jQuery.ajax({
								type: "POST",
								url: "<?php 
        echo admin_url( 'admin-ajax.php' );
        ?>",
								data: { 		
									action: 'generate_organizer_description_with_oai', // call function "generate_organizer_description_with_oai"					
									_ajax_nonce: '<?php 
        echo esc_attr( $nonce );
        ?>',
									searchstring: searchstring,
									existing_content: existing_content,
									url: url
								},
								beforeSend: function() {
									jQuery("#status_generate_event_description_single_organizer").fadeIn('fast');  // Fade in status when button is clicked							
								},
								success : function( data ) {
									info = JSON.parse( data );

									// Error handling
									if ( info.error_message ) {
										alert("Status: "+info.status+"\nError message: "+info.error_message);
									} else {
										// Check if info.description is defined and not empty
										if (info.description && info.description.trim() !== "") {
											// Add generated description to content field (either TinymCE or text editor)
											if (typeof tinyMCE !== "undefined" && tinyMCE.activeEditor != null && !tinyMCE.activeEditor.isHidden()) {
												tinyMCE.activeEditor.setContent(info.description);
											} else if (jQuery("#content").length > 0) {
												jQuery("#content").val(info.description);
											} else {
												alert("Unable to set content: No suitable editor or text area found");
											}
										} else {
											alert("No description retrieved. Please check the search string and try again.");
										}
									}
									
									jQuery("#generate_organizer_description_button").attr("disabled", false); // Enable button after
									jQuery("#status_generate_event_description_single_organizer").fadeOut('slow');	// Fade out status after data is saved
								}
							})
						}
						return false;					
					});											
				});
				
			</script>

			<style type='text/css'>
				#status_generate_event_description_single_organizer { 
					display: none;
					clear: both;
					background: url(<?php 
        echo esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' );
        ?>) left top no-repeat; 
					text-align: left;
					vertical-align: center;	
					padding-left: 25px;
				}
			</style>

			<?php 
    }
}

//----------------------------------------------------------------------------------------
// Generate organizer description and hand it over to Java Script function above
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_generate_organizer_description_with_oai', 'generate_organizer_description_with_open_ai' );
function generate_organizer_description_with_open_ai() {
}

//----------------------------------------------------------------------------------------
// Fetch data from Google Places for single organizer
//----------------------------------------------------------------------------------------
add_action( 'tribe_events_organizer_before_metabox', 'fetch_organizer_data' );
function fetch_organizer_data() {
}

//----------------------------------------------------------------------------------------
// Get organizer data from Google Places and hand it over to Java Script function above
// https://www.codexworld.com/post-get-json-data-from-php-script-jquery-ajax/
// https://www.mediaevent.de/javascript/json.html
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_fetch_organizer_data_from_google_places', 'fetch_organizer_data_from_google_places' );
function fetch_organizer_data_from_google_places() {
}

//----------------------------------------------------------------------------------------
// Events with invalid data
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_events_wid', 'events_with_invalid_data' );
function events_with_invalid_data() {
    // Check the nonce
    if ( !check_ajax_referer( 'events-optimizer-nonce', '_wpnonce', false ) ) {
        wp_die( 'Invalid nonce' );
    }
    // Maximum number of results to display per page
    $max_results_per_page = intval( get_option( 'teo_max_results_per_page', 5 ) );
    // Objects to be ignored while searching for invalid data
    $ignored_and_added_objects = maybe_unserialize( get_option( 'teo_ignored_and_added_objects' ) );
    if ( empty( $ignored_and_added_objects ) ) {
        $ignored_and_added_objects = array();
    }
    // Whether ignored objects should be shown or not
    $show_ignored_objects = get_option( 'teo_show_ignored_objects' );
    // Whether added objects should be shown or not
    $show_added_objects = get_option( 'teo_show_added_objects' );
    // Fields to be ignored while searching for invalid data
    $ignored_field_event_description = get_option( 'teo_ignored_field_event_description' );
    $ignored_field_event_start = get_option( 'teo_ignored_field_event_start' );
    $ignored_field_event_end = get_option( 'teo_ignored_field_event_end' );
    $ignored_field_event_venue = get_option( 'teo_ignored_field_event_venue' );
    $ignored_field_event_organizer = get_option( 'teo_ignored_field_event_organizer' );
    $ignored_field_event_website = get_option( 'teo_ignored_field_event_website' );
    $ignored_field_event_currency_symbol = get_option( 'teo_ignored_field_event_currency_symbol' );
    $ignored_field_event_cost = get_option( 'teo_ignored_field_event_cost' );
    // Get custom currency symbols from Events Calendar Options
    $options = get_option( 'tribe_events_calendar_options', '' );
    if ( !empty( $options['tribe_ext_ce_cost_currency_symbols'] ) ) {
        $currency_symbols_arr = explode( '|', $options['tribe_ext_ce_cost_currency_symbols'] );
    } else {
        $currency_symbols_arr = array();
    }
    foreach ( $currency_symbols_arr as $i => $symbol ) {
        $currency_symbols_arr[$i] = sanitize_text_field( $symbol );
    }
    if ( empty( array_filter( $currency_symbols_arr ) ) ) {
        if ( !empty( $options['defaultCurrencySymbol'] ) ) {
            $currency_symbols_arr[0] = sanitize_text_field( $options['defaultCurrencySymbol'] );
        }
    }
    // Get existing venues from Events Calendar and create sorted array for dropdown list
    $venues_unordered_arr = tribe_get_venues();
    foreach ( $venues_unordered_arr as $venue ) {
        $venues_arr[] = array(
            'ID'   => intval( $venue->ID ),
            'Name' => sanitize_text_field( $venue->post_title ),
        );
    }
    setlocale( LC_COLLATE, get_locale() );
    $name_arr = array_column( $venues_arr, 'Name' );
    array_multisort(
        $name_arr,
        SORT_ASC,
        SORT_LOCALE_STRING,
        $venues_arr
    );
    // Get existing organizers from Events Calendar and create sorted array for dropdown list
    $organizers_unordered_arr = tribe_get_organizers();
    foreach ( $organizers_unordered_arr as $organizer ) {
        $organizers_arr[] = array(
            'ID'   => intval( $organizer->ID ),
            'Name' => sanitize_text_field( $organizer->post_title ),
        );
    }
    unset($name_arr);
    unset($name_lowercase_arr);
    $name_arr = array_column( $organizers_arr, 'Name' );
    array_multisort(
        $name_arr,
        SORT_ASC,
        SORT_LOCALE_STRING,
        $organizers_arr
    );
    $fields_arr = array(
        'ID'                   => esc_html__( 'ID', 'events-optimizer' ),
        'post_title'           => esc_html__( 'Name', 'events-optimizer' ),
        'post_content'         => esc_html__( 'Description', 'events-optimizer' ),
        '_EventStartDate'      => esc_html__( 'Start', 'events-optimizer' ),
        '_EventEndDate'        => esc_html__( 'End', 'events-optimizer' ),
        '_EventVenueID'        => esc_html__( 'Venue', 'events-optimizer' ),
        '_EventOrganizerID'    => esc_html__( 'Organizer', 'events-optimizer' ),
        '_EventURL'            => esc_html__( 'Website', 'events-optimizer' ),
        '_EventCost'           => esc_html__( 'Cost', 'events-optimizer' ),
        '_EventCurrencySymbol' => esc_html__( 'Currency Symbol', 'events-optimizer' ),
    );
    // Get the next index of the venue if function is called by "Show more" button
    $next_result = ( intval( filter_input( INPUT_POST, 'next_result', FILTER_SANITIZE_NUMBER_INT ) ) ?: 0 );
    $k = $next_result;
    // Set index to the next upcoming venue
    $i = 0;
    // Counter for maximum results per page
    $events_offset = array_count_key( $ignored_and_added_objects, 'tribe_events' );
    $event = tribe_get_events( [
        'posts_per_page' => $next_result + $events_offset + 3 * $max_results_per_page,
    ] );
    $max_events = wp_count_posts( 'tribe_events' )->publish;
    // Check if not all venues have been shown yet and maximum results has not been reached
    while ( $k < $max_events and $i < $max_results_per_page ) {
        // Hide venues that have no errors
        $found_error = false;
        $event_details = get_post_meta( $event[$k]->ID ) + array(
            'ID'                => array($event[$k]->ID),
            'post_title'        => array($event[$k]->post_title),
            'post_content'      => array($event[$k]->post_content),
            '_EventVenueID'     => array($event[$k]->_EventVenueID),
            '_EventOrganizerID' => array($event[$k]->_EventOrganizerID),
        );
        // If event isn't part of the ignored or added objects list show it
        if ( array_search( $event[$k]->ID, array_column( $ignored_and_added_objects, 'post_id' ) ) === false or $show_ignored_objects === 'true' or $show_added_objects === 'true' ) {
            echo '<form action="" method="POST" id="save_event_wid_form_' . esc_attr( $k ) . '">
				<br>
				<table class="widefat fixed striped" >
				<thead>
				<tr>
				<th>' . esc_html__( 'Field', 'events-optimizer' ) . '</th>
				<th>' . esc_html__( 'Value', 'events-optimizer' ) . '</th>
				<th>' . esc_html__( 'Status', 'events-optimizer' ) . '</th>
				</tr>
				</thead>
				<tbody>';
            foreach ( $fields_arr as $field => $fieldname ) {
                if ( isset( $event_details[$field][0] ) ) {
                    $value = $event_details[$field][0];
                } else {
                    $value = '';
                }
                switch ( $field ) {
                    case 'ID':
                        echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="hidden" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '"/><a href="/wp-admin/post.php?post=' . $event[$k]->ID . '&action=edit" target="_blank" rel="noopener noreferrer">' . esc_attr( $value ) . ' <img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png' ) . '" class="external-link-icon" alt="External Link"/></a></td><td>--</td></tr>';
                        break;
                    case 'post_title':
                        if ( empty( $value ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $event_details['post_title'][0] ) . '" target="_blank" rel="noopener noreferrer"> <img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . $valid . '</td></tr>';
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $event_details['post_title'][0] ) . '" target="_blank" rel="noopener noreferrer"> <img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    case 'post_content':
                        if ( empty( $value ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><textarea name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" rows="8" cols="30"></textarea></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                            if ( $ignored_field_event_description === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><textarea name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" rows="8" cols="30">' . esc_textarea( $value ) . '</textarea></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    case '_EventStartDate':
                    case '_EventEndDate':
                        if ( $field === '_EventEndDate' and $value === '' ) {
                            $value = $event_details['_EventStartDate'][0];
                        }
                        if ( $field === '_EventEndDate' and $value === $event_details['_EventStartDate'][0] ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . date_format( date_create( $value ), "Y-m-d H:i" ) . '" class="flexible-textfield"/></td><td><span class="field-status-warning">' . esc_html__( 'End is equal with start. Please check!', 'events-optimizer' ) . '</span></td></tr>';
                            break;
                        }
                        if ( empty( $value ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . date_format( date_create( $value ), "Y-m-d H:i" ) . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                            if ( $field === '_EventStartDate' and $ignored_field_event_start === 'false' ) {
                                $found_error = true;
                            }
                            if ( $field === '_EventEndDate' and $ignored_field_event_end === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . date_format( date_create( $value ), "Y-m-d H:i" ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    case '_EventVenueID':
                        $venues_list = '<select name="_EventVenueID">';
                        $venue_found = false;
                        foreach ( $venues_arr as $venue ) {
                            if ( intval( $value ) === intval( $venue['ID'] ) ) {
                                $venues_list .= '<option value="' . esc_attr( $venue['ID'] ) . '" selected>' . esc_attr( $venue['Name'] ) . '</option>';
                                $venue_found = true;
                            } else {
                                $venues_list .= '<option value="' . esc_attr( $venue['ID'] ) . '">' . esc_attr( $venue['Name'] ) . '</option>';
                            }
                        }
                        if ( $venue_found ) {
                            $venues_list .= '</select>';
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td>' . $venues_list . '</td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        } else {
                            $venues_list .= '<option value="" selected>--</option>';
                            $venues_list .= '</select>';
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td>' . $venues_list . '</td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                            if ( $ignored_field_event_venue === 'false' ) {
                                $found_error = true;
                            }
                        }
                        break;
                    case '_EventOrganizerID':
                        $organizers_list = '<select name="_EventOrganizerID">';
                        $organizer_found = false;
                        foreach ( $organizers_arr as $organizer ) {
                            if ( intval( $value ) === intval( $organizer['ID'] ) ) {
                                $organizers_list .= '<option value="' . esc_attr( $organizer['ID'] ) . '" selected>' . esc_attr( $organizer['Name'] ) . '</option>';
                                $organizer_found = true;
                            } else {
                                $organizers_list .= '<option value="' . esc_attr( $organizer['ID'] ) . '">' . esc_attr( $organizer['Name'] ) . '</option>';
                            }
                        }
                        if ( $organizer_found ) {
                            $organizers_list .= '</select>';
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td>' . $organizers_list . '</td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        } else {
                            $organizers_list .= '<option value="" selected>--</option>';
                            $organizers_list .= '</select>';
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td>' . $organizers_list . '</td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                            if ( $ignored_field_event_organizer === 'false' ) {
                                $found_error = true;
                            }
                        }
                        break;
                    case '_EventURL':
                        $valid = url_valid( $value );
                        if ( $valid['status'] != 'OK' ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $event_details['post_title'][0] ) . '" target="_blank" rel="noopener noreferrer"> <img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . esc_attr( $valid['error_message'] ?? '' ) . '</td></tr>';
                            if ( $ignored_field_event_website === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="' . esc_url( $value ) . '" target="_blank" rel="noopener noreferrer"><img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png' ) . '" class="external-link-icon" alt="External Link"/></a></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    // Check if currency symbol is in custom currency symbol list
                    case '_EventCurrencySymbol':
                        if ( empty( $value ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                            if ( $ignored_field_event_currency_symbol === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            // Custom currency symbols available?
                            if ( !empty( $currency_symbols_arr ) ) {
                                if ( !in_array( $value, $currency_symbols_arr ) ) {
                                    echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . esc_html__( 'Currency symbol is not in custom currency symbol list', 'events-optimizer' ) . '</span></td></tr>';
                                    if ( $ignored_field_event_currency_symbol === 'false' ) {
                                        $found_error = true;
                                    }
                                } else {
                                    echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                                }
                            } else {
                                echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                            }
                        }
                        break;
                    case '_EventCost':
                        if ( !is_numeric( $value ) ) {
                            // set the local language for the date from the wordpress settings
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $event_details['post_title'][0] . ' ' . tribe_get_venue( $event[$k]->ID ) . ' ' . date_i18n( get_option( 'date_format' ), strtotime( $event_details['_EventStartDate'][0] ) ) . ' ' . esc_html__( 'Cost', 'events-optimizer' ) ) . '" target="_blank" rel="noopener noreferrer"> <img src="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/search.svg' ) . '" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . esc_html__( 'Field contains invalid characters or is empty', 'events-optimizer' ) . '</span></td></tr>';
                            if ( $ignored_field_event_cost === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                        }
                        break;
                    default:
                        if ( empty( $value ) ) {
                            echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . esc_html__( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>';
                            if ( $field === 'post_title' ) {
                                $found_error = true;
                            }
                        } else {
                            if ( strlen( $value ) <= 4 ) {
                                echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-warning">' . esc_html__( 'Field content is very short', 'events-optimizer' ) . '</span></td></tr>';
                                if ( $field === 'post_title' ) {
                                    $found_error = true;
                                }
                            } else {
                                echo '<tr><td>' . esc_attr( $fieldname ) . '</td><td><input type="text" name="' . esc_attr( $field ) . '" id="' . esc_attr( $field ) . '" value="' . esc_attr( $value ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . esc_html__( 'OK', 'events-optimizer' ) . '</span></td></tr>';
                            }
                        }
                }
                // end switch
            }
            // end foreach
            echo '</tbody>
					</table>	
					<p><input type="button" name="action" class="button button-primary" id="save_event_wid_button_' . esc_attr( $k ) . '" value="' . esc_html__( 'Save changes', 'events-optimizer' ) . '" onclick="save_form(' . "'" . esc_attr( $k ) . "'" . ');" />
					&nbsp;&nbsp;&nbsp;<input type="button" name="action" class="button delete" id="ignore_object_wid_button_' . esc_attr( $k ) . '" value="' . esc_html__( 'Ignore', 'events-optimizer' ) . '" onclick="ignore_object(' . "'" . esc_attr( $k ) . "'" . ');"';
            disabled( !teo_fs()->can_use_premium_code__premium_only() );
            echo '/>
					</p><div id="status_save_event_wid_' . esc_attr( $k ) . '">' . esc_html__( 'SAVING', 'events-optimizer' ) . '</div></p><br>
					</form>';
        }
        // end if array_search(
        $nonce = wp_create_nonce( 'events-optimizer-nonce' );
        ?>	

			<script  type='text/javascript'>
				function save_form(form_id){
					var datastring = jQuery("#save_event_wid_form_"+form_id).serialize(); 
					jQuery("#save_event_wid_button_"+form_id).attr("disabled", true); // Disable button while saving
					jQuery.ajax({
						type: "POST",
						url: "<?php 
        echo admin_url( 'admin-ajax.php' );
        ?>",
						data: { 
							action: 'save_eve',
							_ajax_nonce: '<?php 
        echo esc_attr( $nonce );
        ?>',
							data: datastring,
						},
						beforeSend: function() {
							jQuery("#status_save_event_wid_"+form_id).fadeIn('fast');  // Fade in status when button is clicked
						},
						success: function(html) {
							jQuery("#status_save_event_wid_"+form_id).fadeOut('slow');	// Fade out status after data is saved
							jQuery("#save_event_wid_button_"+form_id).attr("disabled", false); // Enable button after saving
							jQuery("#save_event_wid_form_"+form_id).slideUp(); // Slide organizer form away after saving									
						}
					}); //close jQuery.ajax
					return false;	
				}
				function ignore_object(form_id){
					var datastring = jQuery("#save_event_wid_form_"+form_id).serialize() + '&type=tribe_events&status=ignored';
					jQuery("#ignore_object_wid_button_"+form_id).attr("disabled", true); // Disable button while ignoring
					jQuery.ajax({
						type: "POST",
						url: "<?php 
        echo admin_url( 'admin-ajax.php' );
        ?>",
						data: { 
							action: 'add_obj',
							_ajax_nonce: '<?php 
        echo esc_attr( $nonce );
        ?>',
							data: datastring,
						},
						beforeSend: function() {
						},
						success: function(html) {
							jQuery("#status_save_event_wid_"+form_id).fadeOut('slow');	// Fade out status after data is ignored
							jQuery("#ignore_object_wid_button_"+form_id).attr("disabled", false); // Enable button after ignoring
							jQuery("#save_event_wid_form_"+form_id).slideUp(); // Slide venue form away after ignoring							
						}
					}); //close jQuery.ajax
					return false;	
				}						
			</script>	

			<style type='text/css'>
				#status_save_event_wid_<?php 
        esc_attr_e( $k );
        ?> { 
					display: none;
					clear: both;
					background: url(<?php 
        echo esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' );
        ?>) left top no-repeat; 
					text-align: left;
					vertical-align: center;	
					padding-left: 25px;
				}
				<?php 
        if ( $found_error === false ) {
            echo "#save_event_wid_form_" . esc_attr( $k ) . ", #save_event_wid_button_" . esc_attr( $k ) . " { display: none; }";
        }
        ?>	
			</style>

			<?php 
        // if invalid data is found increase counter for maximum results per page
        if ( $found_error === true ) {
            $i++;
        }
        $k++;
    }
    // end while
    if ( $k < $max_events ) {
        echo '<div id="additional_results_events_wid_' . esc_attr( $k ) . '"><p><strong>' . esc_html__( 'Additional results available.', 'events-optimizer' ) . '</strong></p></div>';
        echo '<p><input type="submit" name="action" class="button button-primary" id="show_more_events_wid_button_' . esc_attr( $k ) . '" value="' . esc_html__( 'Show more', 'events-optimizer' ) . '" onclick="show_more(' . "'" . esc_attr( $k ) . "'" . ');" /></p>';
        echo '<div id="show_more_events_wid"></div>';
        echo '<div id="status_show_more_events_wid_' . esc_attr( $k ) . '">' . esc_html__( 'ANALYZING', 'events-optimizer' ) . '</div>';
    } else {
        echo '<span class="field-status-ok">' . esc_html__( 'No (more) results found!', 'events-optimizer' ) . '</span>';
    }
    $nonce = wp_create_nonce( 'events-optimizer-nonce' );
    ?>	
			
		<script  type='text/javascript'>
		
			function show_more(form_id){
				jQuery("#show_more_events_wid_button_"+form_id).fadeOut('slow'); // Hide button after  		
				jQuery("#show_more_events_wid_button_"+form_id).attr("disabled", true); // Disable button while retrieving more results
				jQuery.ajax({
					type: "POST",
					url: "<?php 
    echo admin_url( 'admin-ajax.php' );
    ?>",
					data: { 
						action: 'events_wid',
						next_result: form_id,
						_ajax_nonce: '<?php 
    echo esc_attr( $nonce );
    ?>' 
					},
					beforeSend: function() {
						jQuery("#status_show_more_events_wid_"+form_id).fadeIn('fast');  // Fade in status when button is clicked
						jQuery("#additional_results_events_wid_"+form_id).fadeOut('slow');	// Fade out text "Additional results ..." after data is saved
					},
					success: function(html) {
						jQuery("#show_more_events_wid_button_"+form_id).attr("disabled", false); // Enable button after
						jQuery("#status_show_more_events_wid_"+form_id).fadeOut('slow');	// Fade out status after data is saved
						jQuery("#show_more_events_wid").append(html); // Add the additional results to page
					}
				}); //close jQuery.ajax
				return false;	
			}	
		</script>

		<style type='text/css'>
			#status_show_more_events_wid_<?php 
    esc_attr_e( $k );
    ?> { 
				display: none;
				clear: both;
				background: url(<?php 
    echo esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' );
    ?>) left top no-repeat; 
				text-align: left;
				vertical-align: center;	
				padding-left: 25px;
			}
		</style>

		<?php 
    wp_die();
}

//----------------------------------------------------------------------------------------
// Save event data (for events with invalid data)
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_save_eve', 'save_event' );
function save_event() {
    // Check the nonce
    if ( !check_ajax_referer( 'events-optimizer-nonce', '_wpnonce', false ) ) {
        wp_die( 'Invalid nonce' );
    }
    parse_str( $_POST['data'], $event );
    // Sanitize each value in the array
    foreach ( $event as $key => $value ) {
        $event[$key] = sanitize_text_field( $value );
    }
    // Title and description can't be updated directly with the TEC API (empty fields gets ignored - bug?) and therefore it needs to be updated seperatly
    $event_post_arr = array(
        'ID'           => $event['ID'],
        'post_title'   => sanitize_text_field( $event['post_title'] ),
        'post_content' => wp_kses_post( $event['post_content'] ),
    );
    wp_update_post( $event_post_arr );
    // Update event with additional values
    $event_arr = array(
        'EventStartDate'      => substr( $event['_EventStartDate'], 0, 10 ),
        'EventEndDate'        => substr( $event['_EventEndDate'], 0, 10 ),
        'EventStartHour'      => substr( $event['_EventStartDate'], 11, 2 ),
        'EventEndHour'        => substr( $event['_EventEndDate'], 11, 2 ),
        'EventStartMinute'    => substr( $event['_EventStartDate'], 14, 2 ),
        'EventEndMinute'      => substr( $event['_EventEndDate'], 14, 2 ),
        'Venue'               => array(
            'VenueID' => sanitize_text_field( $event['_EventVenueID'] ),
        ),
        'Organizer'           => array(
            'OrganizerID' => sanitize_text_field( $event['_EventOrganizerID'] ),
        ),
        'EventURL'            => sanitize_text_field( $event['_EventURL'] ),
        'EventCost'           => sanitize_text_field( $event['_EventCost'] ),
        'EventCurrencySymbol' => sanitize_text_field( $event['_EventCurrencySymbol'] ),
    );
    //	'FeaturedImage'			=> sanitize_text_field( $event[ '_thumbnail_id' ] )
    tribe_update_event( $event['ID'], $event_arr );
    wp_die();
}

//----------------------------------------------------------------------------------------
// Add button to generate description for events, venues and organizers via hook
//----------------------------------------------------------------------------------------
//add_filter( 'edit_form_after_editor', 'add_button_to_generate_description');
//add_filter('media_buttons_context', 'add_button_to_generate_description');
//add_filter( 'edit_form_after_title', 'add_button_to_generate_description');
//add_action( 'media_buttons', 'add_button_to_generate_description');
//add_action( 'tribe_events_venue_before_metabox', 'add_button_to_generate_description');
//add_action( 'edit_form_before_editor', 'add_button_to_generate_description');
add_action( 'edit_form_after_editor', 'add_button_to_generate_description' );
function add_button_to_generate_description() {
    $screen = get_current_screen();
    echo "<br>";
    switch ( get_current_screen()->post_type ) {
        case 'tribe_events':
            generate_event_description();
            break;
        case 'tribe_venue':
            generate_venue_description();
            break;
        case 'tribe_organizer':
            generate_organizer_description();
            break;
    }
}

//----------------------------------------------------------------------------------------
// Generate description for single event
//----------------------------------------------------------------------------------------
function generate_event_description() {
    // Activate only for Pro version
    if ( teo_fs()->can_use_premium_code__premium_only() and get_option( 'teo_openai_hide_generate_description_buttons' ) === 'false' ) {
        $nonce = wp_create_nonce( 'events-optimizer-nonce' );
        wp_enqueue_style( 'style.css', plugin_dir_url( __FILE__ ) . 'assets/css/style.css' );
        // Add button to existing event form
        ?>	
		
			<input type="button" name="action" id="generate_event_description_button" class="button-primary" value="<?php 
        esc_attr_e( 'Generate description', 'events-optimizer' );
        ?>" />
			<span class="field-tip">
				&nbsp;<span class="question-mark-centered"><img src ="<?php 
        echo esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/question-mark.png' );
        ?>"></span>
				<span class="tip-content"><?php 
        _e( 'Event name, date, time and venue are used to generate the description. Please enter at least the name!', 'events-optimizer' );
        ?></span>&nbsp;&nbsp;
				<span id="status_generate_event_description_single_event"><?php 
        esc_html_e( 'GENERATING', 'events-optimizer' );
        ?></span>
			</span>

			<script type='text/javascript'>

				jQuery(document).ready(function() {

					jQuery("#generate_event_description_button").on('click', function()  {

						jQuery("#generate_event_description_button").attr("disabled", true); // Disable button while retrieving more results
						
						// Get event website URL
						url = document.getElementById("EventURL").value;

						// Get existing content from TinyMCE editor or text editor
						if (typeof tinyMCE !== "undefined" && tinyMCE.activeEditor != null && !tinyMCE.activeEditor.isHidden()) {
							existing_content = tinymce.activeEditor.getContent();
						} else if ( jQuery('#content').length > 0 ) { 
							existing_content = jQuery('#content').val();
						}

						// Get search string from title, date, time and venue
						searchstring = 	escapeHTML( 
							document.getElementById("title").value + " " +	
							document.getElementById("EventStartDate").value + " " +
							document.getElementById("EventStartTime").value + " " +
							document.getElementById("select2-saved_tribe_venue-container").childNodes[1].textContent.trim()
						);

						if ( searchstring.trim() === "" || escapeHTML( document.getElementById("title").value) === "" )  { 
							alert( 'Event name, date, time and venue are used to generate the description. Please fill at least the name!', 'events_optimizer' );
							jQuery("#generate_event_description_button").attr("disabled", false);// Enable button after	
						} else {							
						
							jQuery.ajax({
								type: "POST",
								url: "<?php 
        echo admin_url( 'admin-ajax.php' );
        ?>",
								data: { 		
									action: 'generate_event_description_with_oai', // call function "generate_event_description_with_oai"					
									_ajax_nonce: '<?php 
        echo esc_attr( $nonce );
        ?>',
									searchstring: searchstring,
									existing_content: existing_content,
									url: url
								},
								beforeSend: function() {
									jQuery("#status_generate_event_description_single_event").fadeIn('fast');  // Fade in status when button is clicked					
								},
								success : function( data ) {
									info = JSON.parse(data);

									// Error handling
									if ( info.error_message ) {
										alert("Status: "+info.status+"\nError message: "+info.error_message);									
									} else {
										// Check if info.description is defined and not empty
										if (info.description && info.description.trim() !== "") {
											// Add generated description to content field (either TinymCE or text editor)
											if (typeof tinyMCE !== "undefined" && tinyMCE.activeEditor != null && !tinyMCE.activeEditor.isHidden()) {
												tinyMCE.activeEditor.setContent(info.description);
											} else if (jQuery("#content").length > 0) {
												jQuery("#content").val(info.description);
											} else {
												alert("Unable to set content: No suitable editor or text area found");
											}
										} else {
											alert("No description retrieved. Please check the search string and try again.");
										}
									}
									
									jQuery("#generate_event_description_button").attr("disabled", false); // Enable button after
									jQuery("#status_generate_event_description_single_event").fadeOut('slow');	// Fade out status after data is saved
								}
							})
						}
						return false;					
					});											
				});
				
			</script>

			<style type='text/css'>
				#status_generate_event_description_single_event { 
					display: none;
					clear: both;
					background: url(<?php 
        echo esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' );
        ?>) left top no-repeat; 
					text-align: left;
					vertical-align: center;	
					padding-left: 25px;
				}
			</style>
		
			<?php 
    }
}

//----------------------------------------------------------------------------------------
// Generate event description and hand it over to Java Script function above
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_generate_event_description_with_oai', 'generate_event_description_with_open_ai' );
function generate_event_description_with_open_ai() {
}

//----------------------------------------------------------------------------------------
// Search events from Google data
// Creates a task with the results at DataForSEO
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_search_events_wgd', 'search_events_with_google_data' );
function search_events_with_google_data() {
}

//----------------------------------------------------------------------------------------
// Show search results and add events from DataForSEO (Google events data)
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_add_events_wgd', 'add_events_with_google_data' );
function add_events_with_google_data() {
}

//----------------------------------------------------------------------------------------
// Add event to local database
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_add_eve', 'add_event' );
function add_event() {
}

//----------------------------------------------------------------------------------------
// Add jpg, webp, svg and png to mime types that are allowed to be uploaded
//----------------------------------------------------------------------------------------
add_filter( 'upload_mimes', 'custom_upload_mimes', 1 );
function custom_upload_mimes(  $existing_mimes  ) {
    // add webp, jpeg, jpg and png to the list of mime types
    $existing_mimes['webp'] = 'image/webp';
    $existing_mimes['jpeg'] = 'image/jpeg';
    $existing_mimes['jpg'] = 'image/jpeg';
    $existing_mimes['png'] = 'image/png';
    // return the array back to the function with our added mime type
    return $existing_mimes;
}

//----------------------------------------------------------------------------------------
// Add object to list of added and ignored objects in WordPress options
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_add_obj', 'add_object' );
function add_object() {
}

//----------------------------------------------------------------------------------------
// Remove object from list of added and ignored objects
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_remove_obj', 'remove_object' );
function remove_object() {
}

//----------------------------------------------------------------------------------------
// Remove object(s) from list of added and ignored objects when according post is deleted
//----------------------------------------------------------------------------------------
add_action( 'wp_trash_post', 'wpdocs_trash_multiple_posts' );
function wpdocs_trash_multiple_posts(  $post_id = ''  ) {
}

// Removes object
function remove_object_when_post_deleted(  $post_id  ) {
}

//----------------------------------------------------------------------------------------
// Reset list of added or ignored objects
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_reset_object_li', 'reset_object_list' );
function reset_object_list() {
}

//----------------------------------------------------------------------------------------
// Add images from Google images with DataForSEO
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_add_img', 'add_images' );
function add_images() {
}

//----------------------------------------------------------------------------------------
// Save settings
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_save_set', 'save_settings' );
function save_settings() {
    // Check the nonce
    if ( !check_ajax_referer( 'events-optimizer-nonce', '_wpnonce', false ) ) {
        wp_die( 'Invalid nonce' );
    }
    // Maximum number of results per page should not exceed 20
    $max_results_per_page = min( 20, intval( $_POST['max_results_per_page'] ) );
    update_option( 'teo_max_results_per_page', $max_results_per_page );
    wp_die();
}

//----------------------------------------------------------------------------------------
// Google Places API: Place text search
// https://medium.com/@pauljkbs/simple-way-to-get-started-with-the-google-places-api-and-php-9a1f55f8919e
//----------------------------------------------------------------------------------------
function google_places_text_search(  $query = ''  ) {
}

//----------------------------------------------------------------------------------------
// Google Places API: Place details with place_id
//----------------------------------------------------------------------------------------
function get_place_details(  $place_id = ''  ) {
}

//----------------------------------------------------------------------------------------
// Google Places API: Place details with cid
// Possible Enhancement:
// https://developers.google.com/maps/documentation/places/web-service/place-id#save-id
//----------------------------------------------------------------------------------------
function get_place_details_cid(  $cid = ''  ) {
}

//----------------------------------------------------------------------------------------
// OpenAI API: Automatically generate a text description for venue, organizer or event
//----------------------------------------------------------------------------------------
function generate_description(  $prompt = '', $url = ''  ) {
}

//----------------------------------------------------------------------------------------
// DataForSEO API: Scrape content from a specific website URL
//----------------------------------------------------------------------------------------
function parse_content(  $url  ) {
}

//----------------------------------------------------------------------------------------
// Extract content from a multi-dimensional array
//----------------------------------------------------------------------------------------
function extract_content(  $content_array, $element, &$return = ""  ) {
    if ( !is_array( $content_array ) ) {
        return;
    }
    foreach ( $content_array as $key => $value ) {
        if ( $key == $element && is_array( $value ) ) {
            foreach ( $value as $subvalue ) {
                if ( isset( $subvalue['text'] ) ) {
                    $return .= $subvalue['text'] . "\n";
                }
            }
        } elseif ( is_array( $value ) ) {
            extract_content( $value, $element, $return );
        }
    }
}

//----------------------------------------------------------------------------------------
// Check if URL is valid and reachable
//----------------------------------------------------------------------------------------
function url_valid(  $url  ) {
    // Check URLs with Google Safe Browsing
    $google_safe_browsing = sanitize_text_field( get_option( 'teo_google_safe_browsing', 'false' ) );
    $validate_url_https = sanitize_text_field( get_option( 'teo_validate_url_https', 'false' ) );
    $validate_url_redirected_moved = sanitize_text_field( get_option( 'teo_validate_url_redirected_moved', 'false' ) );
    // If URL is empty give back error
    if ( empty( $url ) ) {
        return array(
            'status'        => 'URL_EMPTY',
            'error_message' => esc_html__( 'URL is empty', 'events-optimizer' ),
        );
    }
    // Is URL syntax valid? Accept as ok, even when http:// or https:// or www is missing
    $url = filter_var( $url, FILTER_SANITIZE_URL );
    // Check on missing https / http and create array with possibly valid URLs
    if ( $validate_url_https === 'false' ) {
        $valid_urls = array($url, 'http://' . $url, 'https://' . $url);
    } else {
        $valid_urls = array($url, 'http://www' . $url, 'https://www' . $url);
    }
    // Check each URL if it is valid and reachable
    foreach ( $valid_urls as $valid_url ) {
        // Check if URL syntax is valid
        if ( filter_var( $valid_url, FILTER_VALIDATE_URL ) ) {
            // Get the headers of the URL
            unset($file_headers);
            $file_headers = @get_headers( $valid_url );
            // Check if URL is redirected or permanently moved
            if ( teo_fs()->can_use_premium_code__premium_only() and $validate_url_redirected_moved === 'true' ) {
                if ( isset( $file_headers[0] ) ) {
                    if ( strpos( $file_headers[0], '301' ) > 0 ) {
                        return array(
                            'status'        => 'URL_MOVED_PERMANENTLY',
                            'error_message' => esc_html__( 'URL is moved permanently', 'events-optimizer' ),
                        );
                    } elseif ( strpos( $file_headers[0], '302' ) > 0 ) {
                        return array(
                            'status'        => 'URL_TEMPORARILY_REDIRECTED',
                            'error_message' => esc_html__( 'URL is temporarily redirected', 'events-optimizer' ),
                        );
                    }
                }
            }
            // Check if URL is reachable (status code <> 200)
            if ( isset( $file_headers[0] ) and (bool) preg_match( '/^HTTP\\/\\d\\.\\d\\s+2\\d{2}\\s+/', $file_headers[0] ) ) {
                // Activate only for Pro version
                if ( teo_fs()->can_use_premium_code__premium_only() and $google_safe_browsing === 'true' ) {
                    // Check if URL is marked as suspicious by Google Safe Browsing
                    $return = url_safe( $valid_url );
                    if ( $return['status'] === 'OK' ) {
                        // URL is valid
                        return array(
                            'status'        => 'OK',
                            'error_message' => esc_html__( 'URL is valid', 'events-optimizer' ),
                        );
                    } else {
                        if ( isset( $return['error_message'] ) ) {
                            return array(
                                'status'        => $return['status'],
                                'error_message' => $return['error_message'],
                            );
                        }
                    }
                } else {
                    // URL is valid
                    return array(
                        'status'        => 'OK',
                        'error_message' => esc_html__( 'URL is valid', 'events-optimizer' ),
                    );
                }
            } else {
                return array(
                    'status'        => 'URL_NOT_REACHABLE',
                    'error_message' => esc_html__( 'URL is not reachable', 'events-optimizer' ),
                );
            }
        } else {
            return array(
                'status'        => 'INVALID_URL_SYNTAX',
                'error_message' => esc_html__( 'URL syntax is not valid', 'events-optimizer' ),
            );
        }
    }
    // end foreach
}

//----------------------------------------------------------------------------------------
// Check if URL is marked as suspicious by Google
// https://transparencyreport.google.com/safe-browsing/search?url
//----------------------------------------------------------------------------------------
// yandex safe browsing test urls
// https://ydx-phish-shavar.cepera.ru
// https://ydx-malware-driveby-shavar.cepera.ru
// google safe browsing test urls
// (see all in: https://testsafebrowsing.appspot.com )
// https://testsafebrowsing.appspot.com/s/phishing.html
// https://testsafebrowsing.appspot.com/s/malware.html
// https://github.com/QDenka/safe-browsing
//----------------------------------------------------------------------------------------
function url_safe(  $url  ) {
    $api_key = sanitize_text_field( get_option( 'teo_google_places_api_key' ) );
    // If API Key is set use Google Safe Browsing V4 API for Pro version
    if ( teo_fs()->can_use_premium_code__premium_only() and !empty( $api_key ) ) {
        $api_url = 'https://safebrowsing.googleapis.com/v4/threatMatches:find?key=' . $api_key;
        $request_data = array(
            'client'     => array(
                'clientId'      => preg_replace( "/[^A-Za-z0-9]/", '', sanitize_text_field( get_bloginfo( 'name' ) ) ),
                'clientVersion' => get_plugin_data( __FILE__ )['Version'],
            ),
            'threatInfo' => array(
                'threatTypes'      => array(
                    'THREAT_TYPE_UNSPECIFIED',
                    'MALWARE',
                    'SOCIAL_ENGINEERING',
                    'UNWANTED_SOFTWARE',
                    'POTENTIALLY_HARMFUL_APPLICATION'
                ),
                'platformTypes'    => array('ANY_PLATFORM'),
                'threatEntryTypes' => array('URL'),
                'threatEntries'    => array(array(
                    'url' => $url,
                )),
            ),
        );
        $response = wp_remote_post( $api_url, array(
            'method'  => 'POST',
            'headers' => array(
                'Content-Type' => 'application/json',
            ),
            'body'    => json_encode( $request_data ),
        ) );
        // Check for errors
        if ( is_wp_error( $response ) ) {
            return array(
                'status'        => $response->get_error_code(),
                'error_message' => $response->get_error_message(),
            );
        }
        // Decode the JSON response
        $response_data = json_decode( wp_remote_retrieve_body( $response ), true );
        // Check if the response contains an error
        if ( isset( $response_data['error'] ) ) {
            return array(
                'status'        => $response_data['error']['code'],
                'error_message' => $response_data['error']['message'],
            );
        }
        // If the response data is empty, the URL is safe
        if ( empty( $response_data ) ) {
            return array(
                'status'        => 'OK',
                'error_message' => '',
            );
        } else {
            return array(
                'status'        => 'SUSPICIOUS',
                'error_message' => esc_html__( 'Google has flagged this URL as', 'events-optimizer' ) . ' ' . $response_data['matches'][0]['threatType'],
            );
        }
    }
}

//----------------------------------------------------------------------------------------
// Check if mail address is valid
//----------------------------------------------------------------------------------------
function mail_valid(  $mail = true  ) {
    // Check if mail is empty
    if ( empty( $mail ) ) {
        return esc_html__( 'Mail is empty!', 'events-optimizer' );
    }
    // Check if mail syntax is valid
    if ( !filter_var( $mail, FILTER_VALIDATE_EMAIL ) ) {
        return esc_html__( 'Mail is not valid', 'events-optimizer' );
    }
    // Check if domain exists and is accepting emails
    /* Is not working reliable
    		list( , $domain) = explode('@', $email);
    		if (! checkdnsrr( $domain, 'MX' ) ) {
    			return esc_html__( 'Domain does not exist or accepting mails.', 'events-optimizer' );
    		}
    		*/
}

//----------------------------------------------------------------------------------------
// Check if phone number is valid
// https://uibakery.io/regex-library/phone-number-php
//----------------------------------------------------------------------------------------
function phone_number_valid(  $number = true  ) {
    // Check if phone number is empty
    if ( empty( $number ) ) {
        return esc_html__( 'Phone number is empty', 'events-optimizer' );
    }
    // Define the regular expression pattern to match a valid international phone number format
    $pattern = '/^\\+?\\d{1,4}?[-.\\s]?\\(?\\d{1,3}?\\)?[-.\\s]?\\d{1,4}[-.\\s]?\\d{1,4}[-.\\s]?\\d{1,9}$/';
    // Check if the given phone number matches the pattern
    if ( preg_match( $pattern, $number ) != 1 ) {
        return esc_html__( 'Phone number is not valid', 'events-optimizer' );
    }
}

//----------------------------------------------------------------------------------------
// Create clean URL for SEO (e.g. to create a SEO optimized event image file name)
// https://stackoverflow.com/questions/4783802/converting-string-into-web-safe-uri
//----------------------------------------------------------------------------------------
function get_clean_url(  $str, $replace = array(), $delimiter = '-'  ) {
    if ( !is_string( $str ) ) {
        return '';
    }
    setlocale( LC_ALL, 'en_US.UTF8' );
    if ( !empty( $replace ) ) {
        $str = str_replace( (array) $replace, ' ', $str );
    }
    $clean = preg_replace( array(
        '/Ä/',
        '/Ö/',
        '/Ü/',
        '/ä/',
        '/ö/',
        '/ü/'
    ), array(
        'Ae',
        'Oe',
        'Ue',
        'ae',
        'oe',
        'ue'
    ), $str );
    $clean = iconv( 'UTF-8', 'ASCII//TRANSLIT', $clean );
    $clean = preg_replace( "/[^a-zA-Z0-9\\/_|+ -]/", '', $clean );
    $clean = strtolower( trim( $clean, '-' ) );
    $clean = preg_replace( "/[\\/_|+ -]+/", $delimiter, $clean );
    return $clean;
}

//----------------------------------------------------------------------------------------
// Count elements in array with specific key
// https://stackoverflow.com/questions/11558397/count-specific-values-in-multidimensional-array/11558780
//----------------------------------------------------------------------------------------
function array_count_key(  $array, $find  ) {
    $i = 0;
    foreach ( $array as $key => $value ) {
        if ( isset( $value['type'] ) && $value['type'] === $find ) {
            $i++;
        }
    }
    return $i;
}

//----------------------------------------------------------------------------------------
// Santize multi dimensional array
//----------------------------------------------------------------------------------------
function sanitize_multi_dimensional_array(  $array  ) {
    foreach ( $array as $key => &$value ) {
        if ( is_array( $value ) ) {
            $value = sanitize_multi_dimensional_array( $value );
        } else {
            // Assumes that the data is text. Adjust the sanitizing function if not.
            $value = sanitize_text_field( $value );
        }
    }
    return $array;
}
